from typing import Any, Final, overload, TypeAlias

from .. import functions as cv2

result: TypeAlias = Any
signature: TypeAlias = Any
matchesGMS: TypeAlias = Any
retval: TypeAlias = Any

class AffineFeature2D(cv2.Feature2D): ...

class BEBLID(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getScaleFactor(self) -> retval:
        """"""

    def setScaleFactor(self, scale_factor) -> None:
        """"""

    def create(self, scale_factor, n_bits=...) -> retval:
        """
        @brief Creates the BEBLID descriptor.
        @param scale_factor Adjust the sampling window around detected keypoints: - <b> 1.00f </b> should be the scale for ORB keypoints - <b> 6.75f </b> should be the scale for SIFT detected keypoints - <b> 6.25f </b> is default and fits for KAZE, SURF detected keypoints - <b> 5.00f </b> should be the scale for AKAZE, MSD, AGAST, FAST, BRISK keypoints
        @param n_bits Determine the number of bits in the descriptor. Should be either BEBLID::SIZE_512_BITS or BEBLID::SIZE_256_BITS.
        """

class BoostDesc(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getScaleFactor(self) -> retval:
        """"""

    def getUseScaleOrientation(self) -> retval:
        """"""

    def setScaleFactor(self, scale_factor) -> None:
        """"""

    def setUseScaleOrientation(self, use_scale_orientation) -> None:
        """"""

    def create(self, desc=..., use_scale_orientation=..., scale_factor=...) -> retval:
        """"""

class BriefDescriptorExtractor(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getDescriptorSize(self) -> retval:
        """"""

    def getUseOrientation(self) -> retval:
        """"""

    def setDescriptorSize(self, bytes) -> None:
        """"""

    def setUseOrientation(self, use_orientation) -> None:
        """"""

    def create(self, bytes=..., use_orientation=...) -> retval:
        """"""

class DAISY(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getH(self) -> retval:
        """"""

    def getInterpolation(self) -> retval:
        """"""

    def getNorm(self) -> retval:
        """"""

    def getQHist(self) -> retval:
        """"""

    def getQRadius(self) -> retval:
        """"""

    def getQTheta(self) -> retval:
        """"""

    def getRadius(self) -> retval:
        """"""

    def getUseOrientation(self) -> retval:
        """"""

    def setH(self, H) -> None:
        """"""

    def setInterpolation(self, interpolation) -> None:
        """"""

    def setNorm(self, norm) -> None:
        """"""

    def setQHist(self, q_hist) -> None:
        """"""

    def setQRadius(self, q_radius) -> None:
        """"""

    def setQTheta(self, q_theta) -> None:
        """"""

    def setRadius(self, radius) -> None:
        """"""

    def setUseOrientation(self, use_orientation) -> None:
        """"""

    def create(self, radius=..., q_radius=..., q_theta=..., q_hist=..., norm=..., H=..., interpolation=..., use_orientation=...) -> retval:
        """"""

class FREAK(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getNOctaves(self) -> retval:
        """"""

    def getOrientationNormalized(self) -> retval:
        """"""

    def getPatternScale(self) -> retval:
        """"""

    def getScaleNormalized(self) -> retval:
        """"""

    def setNOctaves(self, nOctaves) -> None:
        """"""

    def setOrientationNormalized(self, orientationNormalized) -> None:
        """"""

    def setPatternScale(self, patternScale) -> None:
        """"""

    def setScaleNormalized(self, scaleNormalized) -> None:
        """"""

    def create(self, orientationNormalized=..., scaleNormalized=..., patternScale=..., nOctaves=..., selectedPairs=...) -> retval:
        """
        @param orientationNormalized Enable orientation normalization.
        @param scaleNormalized Enable scale normalization.
        @param patternScale Scaling of the description pattern.
        @param nOctaves Number of octaves covered by the detected keypoints.
        @param selectedPairs (Optional) user defined selected pairs indexes,
        """

class HarrisLaplaceFeatureDetector(cv2.Feature2D):
    def getCornThresh(self) -> retval:
        """"""

    def getDOGThresh(self) -> retval:
        """"""

    def getDefaultName(self) -> retval:
        """"""

    def getMaxCorners(self) -> retval:
        """"""

    def getNumLayers(self) -> retval:
        """"""

    def getNumOctaves(self) -> retval:
        """"""

    def setCornThresh(self, corn_thresh_) -> None:
        """"""

    def setDOGThresh(self, DOG_thresh_) -> None:
        """"""

    def setMaxCorners(self, maxCorners_) -> None:
        """"""

    def setNumLayers(self, num_layers_) -> None:
        """"""

    def setNumOctaves(self, numOctaves_) -> None:
        """"""

    def create(self, numOctaves=..., corn_thresh=..., DOG_thresh=..., maxCorners=..., num_layers=...) -> retval:
        """
        * @brief Creates a new implementation instance.
        *
        * @param numOctaves the number of octaves in the scale-space pyramid
        * @param corn_thresh the threshold for the Harris cornerness measure
        * @param DOG_thresh the threshold for the Difference-of-Gaussians scale selection
        * @param maxCorners the maximum number of corners to consider
        * @param num_layers the number of intermediate scales per octave
        """

class LATCH(cv2.Feature2D):
    def getBytes(self) -> retval:
        """"""

    def getDefaultName(self) -> retval:
        """"""

    def getHalfSSDsize(self) -> retval:
        """"""

    def getRotationInvariance(self) -> retval:
        """"""

    def getSigma(self) -> retval:
        """"""

    def setBytes(self, bytes) -> None:
        """"""

    def setHalfSSDsize(self, half_ssd_size) -> None:
        """"""

    def setRotationInvariance(self, rotationInvariance) -> None:
        """"""

    def setSigma(self, sigma) -> None:
        """"""

    def create(self, bytes=..., rotationInvariance=..., half_ssd_size=..., sigma=...) -> retval:
        """"""

class LUCID(cv2.Feature2D):
    def getBlurKernel(self) -> retval:
        """"""

    def getDefaultName(self) -> retval:
        """"""

    def getLucidKernel(self) -> retval:
        """"""

    def setBlurKernel(self, blur_kernel) -> None:
        """"""

    def setLucidKernel(self, lucid_kernel) -> None:
        """"""

    def create(self, lucid_kernel=..., blur_kernel=...) -> retval:
        """
        * @param lucid_kernel kernel for descriptor construction, where 1=3x3, 2=5x5, 3=7x7 and so forth
        * @param blur_kernel kernel for blurring image prior to descriptor construction, where 1=3x3, 2=5x5, 3=7x7 and so forth
        """

class MSDDetector(cv2.Feature2D):
    def getComputeOrientation(self) -> retval:
        """"""

    def getDefaultName(self) -> retval:
        """"""

    def getKNN(self) -> retval:
        """"""

    def getNScales(self) -> retval:
        """"""

    def getNmsRadius(self) -> retval:
        """"""

    def getNmsScaleRadius(self) -> retval:
        """"""

    def getPatchRadius(self) -> retval:
        """"""

    def getScaleFactor(self) -> retval:
        """"""

    def getSearchAreaRadius(self) -> retval:
        """"""

    def getThSaliency(self) -> retval:
        """"""

    def setComputeOrientation(self, compute_orientation) -> None:
        """"""

    def setKNN(self, kNN) -> None:
        """"""

    def setNScales(self, use_orientation) -> None:
        """"""

    def setNmsRadius(self, nms_radius) -> None:
        """"""

    def setNmsScaleRadius(self, nms_scale_radius) -> None:
        """"""

    def setPatchRadius(self, patch_radius) -> None:
        """"""

    def setScaleFactor(self, scale_factor) -> None:
        """"""

    def setSearchAreaRadius(self, use_orientation) -> None:
        """"""

    def setThSaliency(self, th_saliency) -> None:
        """"""

    def create(self, m_patch_radius=..., m_search_area_radius=..., m_nms_radius=..., m_nms_scale_radius=..., m_th_saliency=..., m_kNN=..., m_scale_factor=..., m_n_scales=..., m_compute_orientation=...) -> retval:
        """"""

class PCTSignatures(cv2.Algorithm):
    def computeSignature(self, image, signature=...) -> signature:
        """
        * @brief Computes signature of given image.
        * @param image Input image of CV_8U type.
        * @param signature Output computed signature.
        """

    def computeSignatures(self, images, signatures) -> None:
        """
        * @brief Computes signatures for multiple images in parallel.
        * @param images Vector of input images of CV_8U type.
        * @param signatures Vector of computed signatures.
        """

    def getClusterMinSize(self) -> retval:
        """
        * @brief This parameter multiplied by the index of iteration gives lower limit for cluster size.
        *       Clusters containing fewer points than specified by the limit have their centroid dismissed
        *       and points are reassigned.
        """

    def getDistanceFunction(self) -> retval:
        """
        * @brief Distance function selector used for measuring distance between two points in k-means.
        """

    def getDropThreshold(self) -> retval:
        """
        * @brief Remove centroids in k-means whose weight is lesser or equal to given threshold.
        """

    def getGrayscaleBits(self) -> retval:
        """
        * @brief Color resolution of the greyscale bitmap represented in allocated bits
        *       (i.e., value 4 means that 16 shades of grey are used).
        *       The greyscale bitmap is used for computing contrast and entropy values.
        """

    def getInitSeedCount(self) -> retval:
        """
        * @brief Number of initial seeds (initial number of clusters) for the k-means algorithm.
        """

    def getInitSeedIndexes(self) -> retval:
        """
        * @brief Initial seeds (initial number of clusters) for the k-means algorithm.
        """

    def getIterationCount(self) -> retval:
        """
        * @brief Number of iterations of the k-means clustering.
        *       We use fixed number of iterations, since the modified clustering is pruning clusters
        *       (not iteratively refining k clusters).
        """

    def getJoiningDistance(self) -> retval:
        """
        * @brief Threshold euclidean distance between two centroids.
        *       If two cluster centers are closer than this distance,
        *       one of the centroid is dismissed and points are reassigned.
        """

    def getMaxClustersCount(self) -> retval:
        """
        * @brief Maximal number of generated clusters. If the number is exceeded,
        *       the clusters are sorted by their weights and the smallest clusters are cropped.
        """

    def getSampleCount(self) -> retval:
        """
        * @brief Number of initial samples taken from the image.
        """

    def getSamplingPoints(self) -> retval:
        """
        * @brief Initial samples taken from the image.
        *       These sampled features become the input for clustering.
        """

    def getWeightA(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWeightB(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWeightContrast(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWeightEntropy(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWeightL(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWeightX(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWeightY(self) -> retval:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def getWindowRadius(self) -> retval:
        """
        * @brief Size of the texture sampling window used to compute contrast and entropy
        *       (center of the window is always in the pixel selected by x,y coordinates
        *       of the corresponding feature sample).
        """

    def setClusterMinSize(self, clusterMinSize) -> None:
        """
        * @brief This parameter multiplied by the index of iteration gives lower limit for cluster size.
        *       Clusters containing fewer points than specified by the limit have their centroid dismissed
        *       and points are reassigned.
        """

    def setDistanceFunction(self, distanceFunction) -> None:
        """
        * @brief Distance function selector used for measuring distance between two points in k-means.
        *       Available: L0_25, L0_5, L1, L2, L2SQUARED, L5, L_INFINITY.
        """

    def setDropThreshold(self, dropThreshold) -> None:
        """
        * @brief Remove centroids in k-means whose weight is lesser or equal to given threshold.
        """

    def setGrayscaleBits(self, grayscaleBits) -> None:
        """
        * @brief Color resolution of the greyscale bitmap represented in allocated bits
        *       (i.e., value 4 means that 16 shades of grey are used).
        *       The greyscale bitmap is used for computing contrast and entropy values.
        """

    def setInitSeedIndexes(self, initSeedIndexes) -> None:
        """
        * @brief Initial seed indexes for the k-means algorithm.
        """

    def setIterationCount(self, iterationCount) -> None:
        """
        * @brief Number of iterations of the k-means clustering.
        *       We use fixed number of iterations, since the modified clustering is pruning clusters
        *       (not iteratively refining k clusters).
        """

    def setJoiningDistance(self, joiningDistance) -> None:
        """
        * @brief Threshold euclidean distance between two centroids.
        *       If two cluster centers are closer than this distance,
        *       one of the centroid is dismissed and points are reassigned.
        """

    def setMaxClustersCount(self, maxClustersCount) -> None:
        """
        * @brief Maximal number of generated clusters. If the number is exceeded,
        *       the clusters are sorted by their weights and the smallest clusters are cropped.
        """

    def setSamplingPoints(self, samplingPoints) -> None:
        """
        * @brief Sets sampling points used to sample the input image.
        * @param samplingPoints Vector of sampling points in range [0..1) * @note Number of sampling points must be greater or equal to clusterization seed count.
        """

    def setTranslation(self, idx, value) -> None:
        """
        * @brief Translations of the individual axes of the feature space.
        * @param idx ID of the translation
        * @param value Value of the translation * @note *       WEIGHT_IDX = 0; *       X_IDX = 1; *       Y_IDX = 2; *       L_IDX = 3; *       A_IDX = 4; *       B_IDX = 5; *       CONTRAST_IDX = 6; *       ENTROPY_IDX = 7;
        """

    def setTranslations(self, translations) -> None:
        """
        * @brief Translations of the individual axes of the feature space.
        * @param translations Values of all translations. * @note *       WEIGHT_IDX = 0; *       X_IDX = 1; *       Y_IDX = 2; *       L_IDX = 3; *       A_IDX = 4; *       B_IDX = 5; *       CONTRAST_IDX = 6; *       ENTROPY_IDX = 7;
        """

    def setWeight(self, idx, value) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space.
        * @param idx ID of the weight
        * @param value Value of the weight * @note *       WEIGHT_IDX = 0; *       X_IDX = 1; *       Y_IDX = 2; *       L_IDX = 3; *       A_IDX = 4; *       B_IDX = 5; *       CONTRAST_IDX = 6; *       ENTROPY_IDX = 7;
        """

    def setWeightA(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeightB(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeightContrast(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeightEntropy(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeightL(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeightX(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeightY(self, weight) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space
        *       (x,y = position; L,a,b = color in CIE Lab space; c = contrast. e = entropy)
        """

    def setWeights(self, weights) -> None:
        """
        * @brief Weights (multiplicative constants) that linearly stretch individual axes of the feature space.
        * @param weights Values of all weights. * @note *       WEIGHT_IDX = 0; *       X_IDX = 1; *       Y_IDX = 2; *       L_IDX = 3; *       A_IDX = 4; *       B_IDX = 5; *       CONTRAST_IDX = 6; *       ENTROPY_IDX = 7;
        """

    def setWindowRadius(self, radius) -> None:
        """
        * @brief Size of the texture sampling window used to compute contrast and entropy
        *       (center of the window is always in the pixel selected by x,y coordinates
        *       of the corresponding feature sample).
        """

    @overload
    def create(self, initSampleCount=..., initSeedCount=..., pointDistribution=...) -> retval:
        """
        * @brief Creates PCTSignatures algorithm using sample and seed count.
        *       It generates its own sets of sampling points and clusterization seed indexes.
        * @param initSampleCount Number of points used for image sampling.
        * @param initSeedCount Number of initial clusterization seeds. *       Must be lower or equal to initSampleCount
        * @param pointDistribution Distribution of generated points. Default: UNIFORM. *       Available: UNIFORM, REGULAR, NORMAL. * @return Created algorithm.
        """

    @overload
    def create(self, initSamplingPoints, initSeedCount) -> retval:
        """
        * @brief Creates PCTSignatures algorithm using pre-generated sampling points
        *       and number of clusterization seeds. It uses the provided
        *       sampling points and generates its own clusterization seed indexes.
        * @param initSamplingPoints Sampling points used in image sampling.
        * @param initSeedCount Number of initial clusterization seeds. *       Must be lower or equal to initSamplingPoints.size(). * @return Created algorithm.
        """

    def create(self, initSamplingPoints, initClusterSeedIndexes) -> retval:
        """
        * @brief Creates PCTSignatures algorithm using pre-generated sampling points
        *       and clusterization seeds indexes.
        * @param initSamplingPoints Sampling points used in image sampling.
        * @param initClusterSeedIndexes Indexes of initial clusterization seeds. *       Its size must be lower or equal to initSamplingPoints.size(). * @return Created algorithm.
        """

    def drawSignature(self, source, signature, result=..., radiusToShorterSideRatio=..., borderThickness=...) -> result:
        """
        * @brief Draws signature in the source image and outputs the result.
        *       Signatures are visualized as a circle
        *       with radius based on signature weight
        *       and color based on signature color.
        *       Contrast and entropy are not visualized.
        * @param source Source image.
        * @param signature Image signature.
        * @param result Output result.
        * @param radiusToShorterSideRatio Determines maximal radius of signature in the output image.
        * @param borderThickness Border thickness of the visualized signature.
        """

    def generateInitPoints(self, initPoints, count, pointDistribution) -> None:
        """
        * @brief Generates initial sampling points according to selected point distribution.
        * @param initPoints Output vector where the generated points will be saved.
        * @param count Number of points to generate.
        * @param pointDistribution Point distribution selector. *       Available: UNIFORM, REGULAR, NORMAL. * @note Generated coordinates are in range [0..1)
        """

class PCTSignaturesSQFD(cv2.Algorithm):
    def computeQuadraticFormDistance(self, _signature0, _signature1) -> retval:
        """
        * @brief Computes Signature Quadratic Form Distance of two signatures.
        * @param _signature0 The first signature.
        * @param _signature1 The second signature.
        """

    def computeQuadraticFormDistances(self, sourceSignature, imageSignatures, distances) -> None:
        """
        * @brief Computes Signature Quadratic Form Distance between the reference signature
        *       and each of the other image signatures.
        * @param sourceSignature The signature to measure distance of other signatures from.
        * @param imageSignatures Vector of signatures to measure distance from the source signature.
        * @param distances Output vector of measured distances.
        """

    def create(self, distanceFunction=..., similarityFunction=..., similarityParameter=...) -> retval:
        """
        * @brief Creates the algorithm instance using selected distance function,
        *       similarity function and similarity function parameter.
        * @param distanceFunction Distance function selector. Default: L2 *       Available: L0_25, L0_5, L1, L2, L2SQUARED, L5, L_INFINITY
        * @param similarityFunction Similarity function selector. Default: HEURISTIC *       Available: MINUS, GAUSSIAN, HEURISTIC
        * @param similarityParameter Parameter of the similarity function.
        """

class SURF(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getExtended(self) -> retval:
        """"""

    def getHessianThreshold(self) -> retval:
        """"""

    def getNOctaveLayers(self) -> retval:
        """"""

    def getNOctaves(self) -> retval:
        """"""

    def getUpright(self) -> retval:
        """"""

    def setExtended(self, extended) -> None:
        """"""

    def setHessianThreshold(self, hessianThreshold) -> None:
        """"""

    def setNOctaveLayers(self, nOctaveLayers) -> None:
        """"""

    def setNOctaves(self, nOctaves) -> None:
        """"""

    def setUpright(self, upright) -> None:
        """"""

    def create(self, hessianThreshold=..., nOctaves=..., nOctaveLayers=..., extended=..., upright=...) -> retval:
        """
        @param hessianThreshold Threshold for hessian keypoint detector used in SURF.
        @param nOctaves Number of pyramid octaves the keypoint detector will use.
        @param nOctaveLayers Number of octave layers within each octave.
        @param extended Extended descriptor flag (true - use extended 128-element descriptors; false - use 64-element descriptors).
        @param upright Up-right or rotated features flag (true - do not compute orientation of features; false - compute orientation).
        """

class StarDetector(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getLineThresholdBinarized(self) -> retval:
        """"""

    def getLineThresholdProjected(self) -> retval:
        """"""

    def getMaxSize(self) -> retval:
        """"""

    def getResponseThreshold(self) -> retval:
        """"""

    def getSuppressNonmaxSize(self) -> retval:
        """"""

    def setLineThresholdBinarized(self, _lineThresholdBinarized) -> None:
        """"""

    def setLineThresholdProjected(self, _lineThresholdProjected) -> None:
        """"""

    def setMaxSize(self, _maxSize) -> None:
        """"""

    def setResponseThreshold(self, _responseThreshold) -> None:
        """"""

    def setSuppressNonmaxSize(self, _suppressNonmaxSize) -> None:
        """"""

    def create(self, maxSize=..., responseThreshold=..., lineThresholdProjected=..., lineThresholdBinarized=..., suppressNonmaxSize=...) -> retval:
        """"""

class TBMR(AffineFeature2D):
    def getMaxAreaRelative(self) -> retval:
        """"""

    def getMinArea(self) -> retval:
        """"""

    def getNScales(self) -> retval:
        """"""

    def getScaleFactor(self) -> retval:
        """"""

    def setMaxAreaRelative(self, maxArea) -> None:
        """"""

    def setMinArea(self, minArea) -> None:
        """"""

    def setNScales(self, n_scales) -> None:
        """"""

    def setScaleFactor(self, scale_factor) -> None:
        """"""

    def create(self, min_area=..., max_area_relative=..., scale_factor=..., n_scales=...) -> retval:
        """"""

class TEBLID(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def create(self, scale_factor, n_bits=...) -> retval:
        """
        @brief Creates the TEBLID descriptor.
        @param scale_factor Adjust the sampling window around detected keypoints: - <b> 1.00f </b> should be the scale for ORB keypoints - <b> 6.75f </b> should be the scale for SIFT detected keypoints - <b> 6.25f </b> is default and fits for KAZE, SURF detected keypoints - <b> 5.00f </b> should be the scale for AKAZE, MSD, AGAST, FAST, BRISK keypoints
        @param n_bits Determine the number of bits in the descriptor. Should be either TEBLID::SIZE_256_BITS or TEBLID::SIZE_512_BITS.
        """

class VGG(cv2.Feature2D):
    def getDefaultName(self) -> retval:
        """"""

    def getScaleFactor(self) -> retval:
        """"""

    def getSigma(self) -> retval:
        """"""

    def getUseNormalizeDescriptor(self) -> retval:
        """"""

    def getUseNormalizeImage(self) -> retval:
        """"""

    def getUseScaleOrientation(self) -> retval:
        """"""

    def setScaleFactor(self, scale_factor) -> None:
        """"""

    def setSigma(self, isigma) -> None:
        """"""

    def setUseNormalizeDescriptor(self, dsc_normalize) -> None:
        """"""

    def setUseNormalizeImage(self, img_normalize) -> None:
        """"""

    def setUseScaleOrientation(self, use_scale_orientation) -> None:
        """"""

    def create(self, desc=..., isigma=..., img_normalize=..., use_scale_orientation=..., scale_factor=..., dsc_normalize=...) -> retval:
        """"""

def BEBLID_create(scale_factor, n_bits=...) -> retval:
    """
    @brief Creates the BEBLID descriptor.
        @param scale_factor Adjust the sampling window around detected keypoints:
        - <b> 1.00f </b> should be the scale for ORB keypoints
        - <b> 6.75f </b> should be the scale for SIFT detected keypoints
        - <b> 6.25f </b> is default and fits for KAZE, SURF detected keypoints
        - <b> 5.00f </b> should be the scale for AKAZE, MSD, AGAST, FAST, BRISK keypoints
        @param n_bits Determine the number of bits in the descriptor. Should be either
         BEBLID::SIZE_512_BITS or BEBLID::SIZE_256_BITS.
    """

def BoostDesc_create(desc=..., use_scale_orientation=..., scale_factor=...) -> retval:
    """
    .
    """

def BriefDescriptorExtractor_create(bytes=..., use_orientation=...) -> retval:
    """
    .
    """

def DAISY_create(radius=..., q_radius=..., q_theta=..., q_hist=..., norm=..., H=..., interpolation=..., use_orientation=...) -> retval:
    """
    .
    """

def FREAK_create(orientationNormalized=..., scaleNormalized=..., patternScale=..., nOctaves=..., selectedPairs=...) -> retval:
    """
    @param orientationNormalized Enable orientation normalization.
        @param scaleNormalized Enable scale normalization.
        @param patternScale Scaling of the description pattern.
        @param nOctaves Number of octaves covered by the detected keypoints.
        @param selectedPairs (Optional) user defined selected pairs indexes,
    """

def HarrisLaplaceFeatureDetector_create(numOctaves=..., corn_thresh=..., DOG_thresh=..., maxCorners=..., num_layers=...) -> retval:
    """
    * @brief Creates a new implementation instance.
         *
         * @param numOctaves the number of octaves in the scale-space pyramid
         * @param corn_thresh the threshold for the Harris cornerness measure
         * @param DOG_thresh the threshold for the Difference-of-Gaussians scale selection
         * @param maxCorners the maximum number of corners to consider
         * @param num_layers the number of intermediate scales per octave
    """

def LATCH_create(bytes=..., rotationInvariance=..., half_ssd_size=..., sigma=...) -> retval:
    """
    .
    """

def LUCID_create(lucid_kernel=..., blur_kernel=...) -> retval:
    """
    * @param lucid_kernel kernel for descriptor construction, where 1=3x3, 2=5x5, 3=7x7 and so forth
         * @param blur_kernel kernel for blurring image prior to descriptor construction, where 1=3x3, 2=5x5, 3=7x7 and so forth
    """

def MSDDetector_create(m_patch_radius=..., m_search_area_radius=..., m_nms_radius=..., m_nms_scale_radius=..., m_th_saliency=..., m_kNN=..., m_scale_factor=..., m_n_scales=..., m_compute_orientation=...) -> retval:
    """
    .
    """

def PCTSignaturesSQFD_create(distanceFunction=..., similarityFunction=..., similarityParameter=...) -> retval:
    """
    * @brief Creates the algorithm instance using selected distance function,
        *       similarity function and similarity function parameter.
        * @param distanceFunction Distance function selector. Default: L2
        *       Available: L0_25, L0_5, L1, L2, L2SQUARED, L5, L_INFINITY
        * @param similarityFunction Similarity function selector. Default: HEURISTIC
        *       Available: MINUS, GAUSSIAN, HEURISTIC
        * @param similarityParameter Parameter of the similarity function.
    """

@overload
def PCTSignatures_create(initSampleCount=..., initSeedCount=..., pointDistribution=...) -> retval:
    """
    * @brief Creates PCTSignatures algorithm using sample and seed count.
        *       It generates its own sets of sampling points and clusterization seed indexes.
        * @param initSampleCount Number of points used for image sampling.
        * @param initSeedCount Number of initial clusterization seeds.
        *       Must be lower or equal to initSampleCount
        * @param pointDistribution Distribution of generated points. Default: UNIFORM.
        *       Available: UNIFORM, REGULAR, NORMAL.
        * @return Created algorithm.
    """

@overload
def PCTSignatures_create(initSampleCount=..., initSeedCount=..., pointDistribution=...) -> retval:
    """
    * @brief Creates PCTSignatures algorithm using pre-generated sampling points
        *       and number of clusterization seeds. It uses the provided
        *       sampling points and generates its own clusterization seed indexes.
        * @param initSamplingPoints Sampling points used in image sampling.
        * @param initSeedCount Number of initial clusterization seeds.
        *       Must be lower or equal to initSamplingPoints.size().
        * @return Created algorithm.
    """

@overload
def PCTSignatures_create(initSampleCount=..., initSeedCount=..., pointDistribution=...) -> retval:
    """
    * @brief Creates PCTSignatures algorithm using pre-generated sampling points
        *       and clusterization seeds indexes.
        * @param initSamplingPoints Sampling points used in image sampling.
        * @param initClusterSeedIndexes Indexes of initial clusterization seeds.
        *       Its size must be lower or equal to initSamplingPoints.size().
        * @return Created algorithm.
    """

def PCTSignatures_drawSignature(source, signature, result=..., radiusToShorterSideRatio=..., borderThickness=...) -> result:
    """
    * @brief Draws signature in the source image and outputs the result.
        *       Signatures are visualized as a circle
        *       with radius based on signature weight
        *       and color based on signature color.
        *       Contrast and entropy are not visualized.
        * @param source Source image.
        * @param signature Image signature.
        * @param result Output result.
        * @param radiusToShorterSideRatio Determines maximal radius of signature in the output image.
        * @param borderThickness Border thickness of the visualized signature.
    """

def PCTSignatures_generateInitPoints(initPoints, count, pointDistribution) -> None:
    """
    * @brief Generates initial sampling points according to selected point distribution.
        * @param initPoints Output vector where the generated points will be saved.
        * @param count Number of points to generate.
        * @param pointDistribution Point distribution selector.
        *       Available: UNIFORM, REGULAR, NORMAL.
        * @note Generated coordinates are in range [0..1)
    """

def SIFT_create(nfeatures=..., nOctaveLayers=..., contrastThreshold=..., edgeThreshold=..., sigma=...) -> retval:
    """
    Use cv.SIFT_create() instead
    """

@overload
def SURF_create(hessianThreshold=..., nOctaves=..., nOctaveLayers=..., extended=..., upright=...) -> retval:
    """
    @param hessianThreshold Threshold for hessian keypoint detector used in SURF.
        @param nOctaves Number of pyramid octaves the keypoint detector will use.
        @param nOctaveLayers Number of octave layers within each octave.
        @param extended Extended descriptor flag (true - use extended 128-element descriptors; false - use
        64-element descriptors).
        @param upright Up-right or rotated features flag (true - do not compute orientation of features;
    """

@overload
def SURF_create(hessianThreshold=..., nOctaves=..., nOctaveLayers=..., extended=..., upright=...) -> retval:
    """ """

def StarDetector_create(maxSize=..., responseThreshold=..., lineThresholdProjected=..., lineThresholdBinarized=..., suppressNonmaxSize=...) -> retval:
    """
    .
    """

def TBMR_create(min_area=..., max_area_relative=..., scale_factor=..., n_scales=...) -> retval:
    """
    .
    """

def TEBLID_create(scale_factor, n_bits=...) -> retval:
    """
    @brief Creates the TEBLID descriptor.
        @param scale_factor Adjust the sampling window around detected keypoints:
        - <b> 1.00f </b> should be the scale for ORB keypoints
        - <b> 6.75f </b> should be the scale for SIFT detected keypoints
        - <b> 6.25f </b> is default and fits for KAZE, SURF detected keypoints
        - <b> 5.00f </b> should be the scale for AKAZE, MSD, AGAST, FAST, BRISK keypoints
        @param n_bits Determine the number of bits in the descriptor. Should be either
         TEBLID::SIZE_256_BITS or TEBLID::SIZE_512_BITS.
    """

def VGG_create(desc=..., isigma=..., img_normalize=..., use_scale_orientation=..., scale_factor=..., dsc_normalize=...) -> retval:
    """
    .
    """

def matchGMS(size1, size2, keypoints1, keypoints2, matches1to2, withRotation=..., withScale=..., thresholdFactor=...) -> matchesGMS:
    """
    @brief GMS (Grid-based Motion Statistics) feature matching strategy described in @cite Bian2017gms .
        @param size1 Input size of image1.
        @param size2 Input size of image2.
        @param keypoints1 Input keypoints of image1.
        @param keypoints2 Input keypoints of image2.
        @param matches1to2 Input 1-nearest neighbor matches.
        @param matchesGMS Matches returned by the GMS matching strategy.
        @param withRotation Take rotation transformation into account.
        @param withScale Take scale transformation into account.
        @param thresholdFactor The higher, the less matches.
        @note
            Since GMS works well when the number of features is large, we recommend to use the ORB feature and set FastThreshold to 0 to get as many as possible features quickly.
            If matching results are not satisfying, please add more features. (We use 10000 for images with 640 X 480).
            If your images have big rotation and scale changes, please set withRotation or withScale to true.
    """

def matchLOGOS(keypoints1, keypoints2, nn1, nn2, matches1to2) -> None:
    """
    @brief LOGOS (Local geometric support for high-outlier spatial verification) feature matching strategy described in @cite Lowry2018LOGOSLG .
        @param keypoints1 Input keypoints of image1.
        @param keypoints2 Input keypoints of image2.
        @param nn1 Index to the closest BoW centroid for each descriptors of image1.
        @param nn2 Index to the closest BoW centroid for each descriptors of image2.
        @param matches1to2 Matches returned by the LOGOS matching strategy.
        @note
            This matching strategy is suitable for features matching against large scale database.
            First step consists in constructing the bag-of-words (BoW) from a representative image database.
            Image descriptors are then represented by their closest codevector (nearest BoW centroid).
    """

BEBLID_SIZE_256_BITS: Final[int]
BEBLID_SIZE_512_BITS: Final[int]
DAISY_NRM_FULL: Final[int]
DAISY_NRM_NONE: Final[int]
DAISY_NRM_PARTIAL: Final[int]
DAISY_NRM_SIFT: Final[int]
PCTSIGNATURES_GAUSSIAN: Final[int]
PCTSIGNATURES_HEURISTIC: Final[int]
PCTSIGNATURES_L0_25: int
PCTSIGNATURES_L0_5: int
PCTSIGNATURES_L1: Final[int]
PCTSIGNATURES_L2: Final[int]
PCTSIGNATURES_L2SQUARED: Final[int]
PCTSIGNATURES_L5: Final[int]
PCTSIGNATURES_L_INFINITY: Final[int]
PCTSIGNATURES_MINUS: Final[int]
PCTSIGNATURES_NORMAL: Final[int]
PCTSIGNATURES_REGULAR: Final[int]
PCTSIGNATURES_UNIFORM: Final[int]
PCTSignatures_GAUSSIAN: Final[int]
PCTSignatures_HEURISTIC: Final[int]
PCTSignatures_L0_25: int
PCTSignatures_L0_5: int
PCTSignatures_L1: Final[int]
PCTSignatures_L2: Final[int]
PCTSignatures_L2SQUARED: Final[int]
PCTSignatures_L5: Final[int]
PCTSignatures_L_INFINITY: Final[int]
PCTSignatures_MINUS: Final[int]
PCTSignatures_NORMAL: Final[int]
PCTSignatures_REGULAR: Final[int]
PCTSignatures_UNIFORM: Final[int]
TEBLID_SIZE_256_BITS: Final[int]
TEBLID_SIZE_512_BITS: Final[int]



from typing import Final

MM_AFFINE: Final[int]
MM_HOMOGRAPHY: Final[int]
MM_RIGID: Final[int]
MM_ROTATION: Final[int]
MM_SIMILARITY: Final[int]
MM_TRANSLATION: Final[int]
MM_TRANSLATION_AND_SCALE: Final[int]
MM_UNKNOWN: Final[int]



from typing import Any, TypeAlias

from .. import functions as cv2

boundingBox: TypeAlias = Any

retval: TypeAlias = Any

class MultiTracker(cv2.Algorithm):
    def add(self, newTracker, image, boundingBox) -> retval:
        """
        * \brief Add a new object to be tracked.
        *
        * @param newTracker tracking algorithm to be used
        * @param image input image
        * @param boundingBox a rectangle represents ROI of the tracked object
        """

    def getObjects(self) -> retval:
        """
        * \brief Returns a reference to a storage for the tracked objects, each object corresponds to one tracker algorithm
        """

    def update(self, image) -> tuple[retval, boundingBox]:
        """
        * \brief Update the current tracking status.
        * @param image input image
        * @param boundingBox the tracking result, represent a list of ROIs of the tracked objects.
        """

    def create(self) -> retval:
        """
        * \brief Returns a pointer to a new instance of MultiTracker
        """

class Tracker(cv2.Algorithm):
    def init(self, image, boundingBox) -> retval:
        """
        @brief Initialize the tracker with a known bounding box that surrounded the target
        @param image The initial frame
        @param boundingBox The initial bounding box  @return True if initialization went succesfully, false otherwise
        """

    def update(self, image) -> tuple[retval, boundingBox]:
        """
        @brief Update the tracker, find the new most likely bounding box for the target
        @param image The current frame
        @param boundingBox The bounding box that represent the new target location, if true was returned, not modified otherwise  @return True means that target was located and false means that tracker cannot locate target in current frame. Note, that latter *does not* imply that tracker has failed, maybe target is indeed missing from the frame (say, out of sight)
        """

class TrackerBoosting(Tracker):
    def create(self) -> retval:
        """
        @brief Constructor
        @param parameters BOOSTING parameters TrackerBoosting::Params
        """

class TrackerCSRT(Tracker):
    def setInitialMask(self, mask) -> None:
        """"""

    def create(self) -> retval:
        """
        @brief Constructor
        @param parameters CSRT parameters TrackerCSRT::Params
        """

class TrackerKCF(Tracker):
    def create(self) -> retval:
        """
        @brief Constructor
        @param parameters KCF parameters TrackerKCF::Params
        """

class TrackerMIL(Tracker):
    def create(self) -> retval:
        """
        @brief Constructor
        @param parameters MIL parameters TrackerMIL::Params
        """

class TrackerMOSSE(Tracker):
    def create(self) -> retval:
        """
        @brief Constructor
        """

class TrackerMedianFlow(Tracker):
    def create(self) -> retval:
        """
        @brief Constructor
        @param parameters Median Flow parameters TrackerMedianFlow::Params
        """

class TrackerTLD(Tracker):
    def create(self) -> retval:
        """
        @brief Constructor
        @param parameters TLD parameters TrackerTLD::Params
        """

def MultiTracker_create() -> retval:
    """
    * \brief Returns a pointer to a new instance of MultiTracker
    """

def TrackerBoosting_create() -> retval:
    """
    @brief Constructor
        @param parameters BOOSTING parameters TrackerBoosting::Params
    """

def TrackerCSRT_create() -> retval:
    """
    @brief Constructor
      @param parameters CSRT parameters TrackerCSRT::Params
    """

def TrackerKCF_create() -> retval:
    """
    @brief Constructor
      @param parameters KCF parameters TrackerKCF::Params
    """

def TrackerMIL_create() -> retval:
    """
    @brief Constructor
        @param parameters MIL parameters TrackerMIL::Params
    """

def TrackerMOSSE_create() -> retval:
    """
    @brief Constructor
    """

def TrackerMedianFlow_create() -> retval:
    """
    @brief Constructor
        @param parameters Median Flow parameters TrackerMedianFlow::Params
    """

def TrackerTLD_create() -> retval:
    """
    @brief Constructor
        @param parameters TLD parameters TrackerTLD::Params
    """

def upgradeTrackingAPI(legacy_tracker) -> retval:
    """
    .
    """



import argparse
from pathlib import Path


def main() -> None:
    parser = argparse.ArgumentParser(description="Script to merge stubs with the Microsoft opencv stubs.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--microsoft_stubs_path", "-m", type=Path, default=Path("./microst_stubs.pyi"),
                        help="Path to pre-existing stubs.")
    parser.add_argument("--new_stubs_path", "-n", type=Path, default=Path("src/cv2-stubs/__init__.pyi"),
                        help="Path to my stubs.")
    args = parser.parse_args()

    microsoft_stubs: Path = args.microsoft_stubs_path
    my_stubs: Path = args.new_stubs_path

    print("Reading pre-existing stubs")
    functions: dict[str, str] = {}  # function_signature: doctring
    done_functions_names: set[str] = set()
    head: list[str] = []
    with my_stubs.open("r", encoding="utf-8") as stubs:
        stub_lines = stubs.readlines()
    for i in range(len(stub_lines)):
        if stub_lines[i][:3] == "def":
            function_name = stub_lines[i][4:].split("(")[0]
            done_functions_names.add(function_name)
            functions[stub_lines[i]] = stub_lines[i+1]

        if len(done_functions_names) == 0:
            head.append(stub_lines[i])
    print(f"Found {len(done_functions_names)} functions.")

    print("Reading Microsoft stubs")
    with microsoft_stubs.open("r", encoding="utf-8") as stubs:
        stub_lines = stubs.readlines()
    for i in range(len(stub_lines)):
        if stub_lines[i][:3] == "def" and stub_lines[i][4:].split("(")[0] not in done_functions_names:
            functions[stub_lines[i]] = stub_lines[i+1]

    print("Writing new stubs")
    with my_stubs.open("w", encoding="utf-8") as new_file:
        new_file.writelines(head[:-1])

        functions = dict(sorted(functions.items(), key=lambda item: item[0]))
        for signature, docstring in functions.items():
            new_file.write("\n")
            new_file.write(signature)
            new_file.write(docstring)
            new_file.write("    ...\n")

    print("Done")

    # Replace commands that can be used to add some typings automatically:
    # sed -i "s/src: Mat/src: npt.NDArray[TImg]/" src/cv2-stubs/__init__.pyi
    # sed -i "s/dst: Mat/dst: npt.NDArray[TImg]/" src/cv2-stubs/__init__.pyi
    # sed -i "s/\(dst: npt.NDArray\[TImg\].*\) -> typing.Any:/\1 -> npt.NDArray[TImg]:/" src/cv2-stubs/__init__.pyi
    # sed -i "s/(image: Mat/(image: npt.NDArray[TImg]/" src/cv2-stubs/__init__.pyi
    # sed -i "s/borderMode=.../borderMode: int = .../" src/cv2-stubs/__init__.pyi
    # sed -i "s/borderValue=.../borderValue: TColor  = .../" src/cv2-stubs/__init__.pyi
    # sed -i "s/src1: Mat/src1: npt.NDArray[TImg]/" src/cv2-stubs/__init__.pyi
    # sed -i "s/src2: Mat/src2: npt.NDArray[TImg]/" src/cv2-stubs/__init__.pyi
    # sed -i "s/typing.Tuple\[/tuple\[/" src/cv2-stubs/__init__.pyi
    # sed -i "s/mask: Mat/mask: TMask/g" src/cv2-stubs/__init__.pyi


if __name__ == "__main__":
    main()



import builtins
from typing import Any, Final, overload, TypeAlias

from .. import functions as cv2

status: TypeAlias = Any
nextPts: TypeAlias = Any
flow: TypeAlias = Any
err: TypeAlias = Any
retval: TypeAlias = Any

class DenseRLOFOpticalFlow(cv2.DenseOpticalFlow):
    def getEPICK(self) -> retval:
        """
        K is a number of nearest-neighbor matches considered, when fitting a locally affine
        *    model. Usually it should be around 128. However, lower values would make the interpolation noticeably faster.
        *    @see ximgproc::EdgeAwareInterpolator,  setEPICK
        """

    def getEPICLambda(self) -> retval:
        """
        Lambda is a parameter defining the weight of the edge-aware term in geodesic distance,
        *    should be in the range of 0 to 1000.
        *    @see ximgproc::EdgeAwareInterpolator, setEPICSigma
        """

    def getEPICSigma(self) -> retval:
        """
        Sigma is a parameter defining how fast the weights decrease in the locally-weighted affine
        *  fitting. Higher values can help preserve fine details, lower values can help to get rid of noise in the
        *  output flow.
        *    @see ximgproc::EdgeAwareInterpolator, setEPICSigma
        """

    def getFgsLambda(self) -> retval:
        """
        Sets the respective fastGlobalSmootherFilter() parameter.
        *    @see ximgproc::EdgeAwareInterpolator, setFgsLambda
        """

    def getFgsSigma(self) -> retval:
        """
        Sets the respective fastGlobalSmootherFilter() parameter.
        *    @see ximgproc::EdgeAwareInterpolator, ximgproc::fastGlobalSmootherFilter, setFgsSigma
        """

    def getForwardBackward(self) -> retval:
        """
        @copybrief setForwardBackward
        @see setForwardBackward
        """

    def getGridStep(self) -> retval:
        """
        For each grid point a motion vector is computed. Some motion vectors will be removed due to the forwatd backward
        *  threshold (if set >0). The rest will be the base of the vector field interpolation.
        *    @see getForwardBackward, setGridStep
        """

    def getInterpolation(self) -> retval:
        """
        @copybrief setInterpolation
        *    @see ximgproc::EdgeAwareInterpolator, setInterpolation
        """

    def getRICSLICType(self) -> retval:
        """
        @copybrief setRICSLICType
        *    @see setRICSLICType
        """

    def getRICSPSize(self) -> retval:
        """
        @copybrief setRICSPSize
        *    @see setRICSPSize
        """

    def getRLOFOpticalFlowParameter(self) -> retval:
        """
        @copybrief setRLOFOpticalFlowParameter
        @see optflow::RLOFOpticalFlowParameter, setRLOFOpticalFlowParameter
        """

    def getUsePostProc(self) -> retval:
        """
        @copybrief setUsePostProc
        *    @see ximgproc::fastGlobalSmootherFilter, setUsePostProc
        """

    def getUseVariationalRefinement(self) -> retval:
        """
        @copybrief setUseVariationalRefinement
        *    @see ximgproc::fastGlobalSmootherFilter, setUsePostProc
        """

    def setEPICK(self, val) -> None:
        """
        @copybrief getEPICK
        *    @see ximgproc::EdgeAwareInterpolator, getEPICK
        """

    def setEPICLambda(self, val) -> None:
        """
        @copybrief getEPICLambda
        *    @see ximgproc::EdgeAwareInterpolator, getEPICLambda
        """

    def setEPICSigma(self, val) -> None:
        """
        @copybrief getEPICSigma
        *  @see ximgproc::EdgeAwareInterpolator, getEPICSigma
        """

    def setFgsLambda(self, val) -> None:
        """
        @copybrief getFgsLambda
        *    @see ximgproc::EdgeAwareInterpolator, ximgproc::fastGlobalSmootherFilter, getFgsLambda
        """

    def setFgsSigma(self, val) -> None:
        """
        @copybrief getFgsSigma
        *    @see ximgproc::EdgeAwareInterpolator, ximgproc::fastGlobalSmootherFilter, getFgsSigma
        """

    def setForwardBackward(self, val) -> None:
        """
        For each grid point \f$ \mathbf{x} \f$ a motion vector \f$ d_{I0,I1}(\mathbf{x}) \f$ is computed.
        *     If the forward backward error \f[ EP_{FB} = || d_{I0,I1} + d_{I1,I0} || \f]
        *     is larger than threshold given by this function then the motion vector will not be used by the following
        *    vector field interpolation. \f$ d_{I1,I0} \f$ denotes the backward flow. Note, the forward backward test
        *    will only be applied if the threshold > 0. This may results into a doubled runtime for the motion estimation.
        *    @see getForwardBackward, setGridStep
        """

    def setGridStep(self, val) -> None:
        """
        @copybrief getGridStep
        *    @see getGridStep
        """

    def setInterpolation(self, val) -> None:
        """
        Two interpolation algorithms are supported
        * - **INTERP_GEO** applies the fast geodesic interpolation, see @cite Geistert2016.
        * - **INTERP_EPIC_RESIDUAL** applies the edge-preserving interpolation, see @cite Revaud2015,Geistert2016.
        * @see ximgproc::EdgeAwareInterpolator, getInterpolation
        """

    def setRICSLICType(self, val) -> None:
        """
        @brief Parameter to choose superpixel algorithm variant to use:
        * - cv::ximgproc::SLICType SLIC segments image using a desired region_size (value: 100)
        * - cv::ximgproc::SLICType SLICO will optimize using adaptive compactness factor (value: 101)
        * - cv::ximgproc::SLICType MSLIC will optimize using manifold methods resulting in more content-sensitive superpixels (value: 102).
        *  @see cv::ximgproc::createSuperpixelSLIC, cv::ximgproc::RICInterpolator
        """

    def setRICSPSize(self, val) -> None:
        """
        * @see cv::ximgproc::createSuperpixelSLIC, cv::ximgproc::RICInterpolator
        """

    def setRLOFOpticalFlowParameter(self, val) -> None:
        """
        @see optflow::RLOFOpticalFlowParameter, getRLOFOpticalFlowParameter
        """

    def setUsePostProc(self, val) -> None:
        """
        * @see getUsePostProc
        """

    def setUseVariationalRefinement(self, val) -> None:
        """
        * @see getUseVariationalRefinement
        """

    def create(self, rlofParam=..., forwardBackwardThreshold=..., gridStep=..., interp_type=..., epicK=..., epicSigma=..., epicLambda=..., ricSPSize=..., ricSLICType=..., use_post_proc=..., fgsLambda=..., fgsSigma=..., use_variational_refinement=...) -> retval:
        """
        *    @param rlofParam see optflow::RLOFOpticalFlowParameter
        *    @param forwardBackwardThreshold see setForwardBackward
        *    @param gridStep see setGridStep
        *    @param interp_type see setInterpolation
        *    @param epicK see setEPICK
        *    @param epicSigma see setEPICSigma
        *    @param epicLambda see setEPICLambda
        *    @param ricSPSize see setRICSPSize
        *    @param ricSLICType see setRICSLICType
        *    @param use_post_proc see setUsePostProc
        *    @param fgsLambda see setFgsLambda
        *    @param fgsSigma see setFgsSigma
        *    @param use_variational_refinement see setUseVariationalRefinement
        """

class DualTVL1OpticalFlow(cv2.DenseOpticalFlow):
    def getEpsilon(self) -> retval:
        """
        @see setEpsilon
        """

    def getGamma(self) -> retval:
        """
        @see setGamma
        """

    def getInnerIterations(self) -> retval:
        """
        @see setInnerIterations
        """

    def getLambda(self) -> retval:
        """
        @see setLambda
        """

    def getMedianFiltering(self) -> retval:
        """
        @see setMedianFiltering
        """

    def getOuterIterations(self) -> retval:
        """
        @see setOuterIterations
        """

    def getScaleStep(self) -> retval:
        """
        @see setScaleStep
        """

    def getScalesNumber(self) -> retval:
        """
        @see setScalesNumber
        """

    def getTau(self) -> retval:
        """
        @see setTau
        """

    def getTheta(self) -> retval:
        """
        @see setTheta
        """

    def getUseInitialFlow(self) -> retval:
        """
        @see setUseInitialFlow
        """

    def getWarpingsNumber(self) -> retval:
        """
        @see setWarpingsNumber
        """

    def setEpsilon(self, val) -> None:
        """
        @copybrief getEpsilon @see getEpsilon
        """

    def setGamma(self, val) -> None:
        """
        @copybrief getGamma @see getGamma
        """

    def setInnerIterations(self, val) -> None:
        """
        @copybrief getInnerIterations @see getInnerIterations
        """

    def setLambda(self, val) -> None:
        """
        @copybrief getLambda @see getLambda
        """

    def setMedianFiltering(self, val) -> None:
        """
        @copybrief getMedianFiltering @see getMedianFiltering
        """

    def setOuterIterations(self, val) -> None:
        """
        @copybrief getOuterIterations @see getOuterIterations
        """

    def setScaleStep(self, val) -> None:
        """
        @copybrief getScaleStep @see getScaleStep
        """

    def setScalesNumber(self, val) -> None:
        """
        @copybrief getScalesNumber @see getScalesNumber
        """

    def setTau(self, val) -> None:
        """
        @copybrief getTau @see getTau
        """

    def setTheta(self, val) -> None:
        """
        @copybrief getTheta @see getTheta
        """

    def setUseInitialFlow(self, val) -> None:
        """
        @copybrief getUseInitialFlow @see getUseInitialFlow
        """

    def setWarpingsNumber(self, val) -> None:
        """
        @copybrief getWarpingsNumber @see getWarpingsNumber
        """

    def create(self, tau=..., lambda_=..., theta=..., nscales=..., warps=..., epsilon=..., innnerIterations=..., outerIterations=..., scaleStep=..., gamma=..., medianFiltering=..., useInitialFlow=...) -> retval:
        """
        @brief Creates instance of cv::DualTVL1OpticalFlow
        """

class GPCDetails: ...
class GPCPatchDescriptor: ...
class GPCPatchSample: ...
class GPCTrainingSamples: ...
class GPCTree(cv2.Algorithm): ...
class OpticalFlowPCAFlow(cv2.DenseOpticalFlow): ...
class PCAPrior: ...

class RLOFOpticalFlowParameter:
    def getCrossSegmentationThreshold(self) -> retval:
        """"""

    def getGlobalMotionRansacThreshold(self) -> retval:
        """"""

    def getLargeWinSize(self) -> retval:
        """"""

    def getMaxIteration(self) -> retval:
        """"""

    def getMaxLevel(self) -> retval:
        """"""

    def getMinEigenValue(self) -> retval:
        """"""

    def getNormSigma0(self) -> retval:
        """"""

    def getNormSigma1(self) -> retval:
        """"""

    def getSmallWinSize(self) -> retval:
        """"""

    def getSolverType(self) -> retval:
        """"""

    def getSupportRegionType(self) -> retval:
        """"""

    def getUseGlobalMotionPrior(self) -> retval:
        """"""

    def getUseIlluminationModel(self) -> retval:
        """"""

    def getUseInitialFlow(self) -> retval:
        """"""

    def setCrossSegmentationThreshold(self, val) -> None:
        """"""

    def setGlobalMotionRansacThreshold(self, val) -> None:
        """"""

    def setLargeWinSize(self, val) -> None:
        """"""

    def setMaxIteration(self, val) -> None:
        """"""

    def setMaxLevel(self, val) -> None:
        """"""

    def setMinEigenValue(self, val) -> None:
        """"""

    def setNormSigma0(self, val) -> None:
        """"""

    def setNormSigma1(self, val) -> None:
        """"""

    def setSmallWinSize(self, val) -> None:
        """"""

    def setSolverType(self, val) -> None:
        """"""

    def setSupportRegionType(self, val) -> None:
        """"""

    def setUseGlobalMotionPrior(self, val) -> None:
        """"""

    def setUseIlluminationModel(self, val) -> None:
        """"""

    def setUseInitialFlow(self, val) -> None:
        """"""

    def setUseMEstimator(self, val) -> None:
        """
        Enables M-estimator by setting sigma parameters to (3.2, 7.0). Disabling M-estimator can reduce
        *  runtime, while enabling can improve the accuracy.
        *  @param val If true M-estimator is used. If false least-square estimator is used. *    @see setNormSigma0, setNormSigma1
        """

    def create(self) -> retval:
        """"""

class SparseRLOFOpticalFlow(cv2.SparseOpticalFlow):
    def getForwardBackward(self) -> retval:
        """
        @copybrief setForwardBackward
        *    @see setForwardBackward
        """

    def getRLOFOpticalFlowParameter(self) -> retval:
        """
        @copybrief setRLOFOpticalFlowParameter
        *    @see setRLOFOpticalFlowParameter
        """

    def setForwardBackward(self, val) -> None:
        """
        For each feature point a motion vector \f$ d_{I0,I1}(\mathbf{x}) \f$ is computed.
        *     If the forward backward error \f[ EP_{FB} = || d_{I0,I1} + d_{I1,I0} || \f]
        *     is larger than threshold given by this function then the status  will not be used by the following
        *    vector field interpolation. \f$ d_{I1,I0} \f$ denotes the backward flow. Note, the forward backward test
        *    will only be applied if the threshold > 0. This may results into a doubled runtime for the motion estimation.
        *    @see setForwardBackward
        """

    def setRLOFOpticalFlowParameter(self, val) -> None:
        """
        @copydoc DenseRLOFOpticalFlow::setRLOFOpticalFlowParameter
        """

    def create(self, rlofParam=..., forwardBackwardThreshold=...) -> retval:
        """
        *    @param rlofParam see setRLOFOpticalFlowParameter
        *    @param forwardBackwardThreshold see setForwardBackward
        """

def DenseRLOFOpticalFlow_create(rlofParam=..., forwardBackwardThreshold=..., gridStep=..., interp_type=..., epicK=..., epicSigma=..., epicLambda=..., ricSPSize=..., ricSLICType=..., use_post_proc=..., fgsLambda=..., fgsSigma=..., use_variational_refinement=...) -> retval:
    """
    *    @param rlofParam see optflow::RLOFOpticalFlowParameter
         *    @param forwardBackwardThreshold see setForwardBackward
         *    @param gridStep see setGridStep
         *    @param interp_type see setInterpolation
         *    @param epicK see setEPICK
         *    @param epicSigma see setEPICSigma
         *    @param epicLambda see setEPICLambda
         *    @param ricSPSize see setRICSPSize
         *    @param ricSLICType see setRICSLICType
         *    @param use_post_proc see setUsePostProc
         *    @param fgsLambda see setFgsLambda
         *    @param fgsSigma see setFgsSigma
         *    @param use_variational_refinement see setUseVariationalRefinement
    """

def DualTVL1OpticalFlow_create(tau=..., lambda_=..., theta=..., nscales=..., warps=..., epsilon=..., innnerIterations=..., outerIterations=..., scaleStep=..., gamma=..., medianFiltering=..., useInitialFlow=...) -> retval:
    """
    @brief Creates instance of cv::DualTVL1OpticalFlow
    """

def RLOFOpticalFlowParameter_create() -> retval:
    """
    .
    """

def SparseRLOFOpticalFlow_create(rlofParam=..., forwardBackwardThreshold=...) -> retval:
    """
    *    @param rlofParam see setRLOFOpticalFlowParameter
         *    @param forwardBackwardThreshold see setForwardBackward
    """

def calcOpticalFlowDenseRLOF(I0, I1, flow, rlofParam=..., forwardBackwardThreshold=..., gridStep=..., interp_type=..., epicK=..., epicSigma=..., epicLambda=..., ricSPSize=..., ricSLICType=..., use_post_proc=..., fgsLambda=..., fgsSigma=..., use_variational_refinement=...) -> flow:
    """
    @brief Fast dense optical flow computation based on robust local optical flow (RLOF) algorithms and sparse-to-dense interpolation scheme.

    The RLOF is a fast local optical flow approach described in @cite Senst2012 @cite Senst2013 @cite Senst2014
    and @cite Senst2016 similar to the pyramidal iterative Lucas-Kanade method as
    proposed by @cite Bouguet00. More details and experiments can be found in the following thesis @cite Senst2019.
    The implementation is derived from optflow::calcOpticalFlowPyrLK().

    The sparse-to-dense interpolation scheme allows for fast computation of dense optical flow using RLOF (see @cite Geistert2016).
    For this scheme the following steps are applied:
    -# motion vector seeded at a regular sampled grid are computed. The sparsity of this grid can be configured with setGridStep
    -# (optinally) errornous motion vectors are filter based on the forward backward confidence. The threshold can be configured
    with setForwardBackward. The filter is only applied if the threshold >0 but than the runtime is doubled due to the estimation
    of the backward flow.
    -# Vector field interpolation is applied to the motion vector set to obtain a dense vector field.

    @param I0 first 8-bit input image. If The cross-based RLOF is used (by selecting optflow::RLOFOpticalFlowParameter::supportRegionType
    = SupportRegionType::SR_CROSS) image has to be a 8-bit 3 channel image.
    @param I1 second 8-bit input image. If The cross-based RLOF is used (by selecting optflow::RLOFOpticalFlowParameter::supportRegionType
    = SupportRegionType::SR_CROSS) image has to be a 8-bit 3 channel image.
    @param flow computed flow image that has the same size as I0 and type CV_32FC2.
    @param rlofParam see optflow::RLOFOpticalFlowParameter
    @param forwardBackwardThreshold Threshold for the forward backward confidence check.
    For each grid point \f$ \mathbf{x} \f$ a motion vector \f$ d_{I0,I1}(\mathbf{x}) \f$ is computed.
    If the forward backward error \f[ EP_{FB} = || d_{I0,I1} + d_{I1,I0} || \f]
    is larger than threshold given by this function then the motion vector will not be used by the following
    vector field interpolation. \f$ d_{I1,I0} \f$ denotes the backward flow. Note, the forward backward test
       will only be applied if the threshold > 0. This may results into a doubled runtime for the motion estimation.
    @param gridStep Size of the grid to spawn the motion vectors. For each grid point a motion vector is computed.
    Some motion vectors will be removed due to the forwatd backward threshold (if set >0). The rest will be the
    base of the vector field interpolation.
    @param interp_type interpolation method used to compute the dense optical flow. Two interpolation algorithms are
    supported:
    - **INTERP_GEO** applies the fast geodesic interpolation, see @cite Geistert2016.
    - **INTERP_EPIC_RESIDUAL** applies the edge-preserving interpolation, see @cite Revaud2015,Geistert2016.
    @param epicK see ximgproc::EdgeAwareInterpolator sets the respective parameter.
    @param epicSigma see ximgproc::EdgeAwareInterpolator sets the respective parameter.
    @param epicLambda see ximgproc::EdgeAwareInterpolator sets the respective parameter.
    @param ricSPSize  see ximgproc::RICInterpolator sets the respective parameter.
    @param ricSLICType see ximgproc::RICInterpolator sets the respective parameter.
    @param use_post_proc enables ximgproc::fastGlobalSmootherFilter() parameter.
    @param fgsLambda sets the respective ximgproc::fastGlobalSmootherFilter() parameter.
    @param fgsSigma sets the respective ximgproc::fastGlobalSmootherFilter() parameter.
    @param use_variational_refinement enables VariationalRefinement

    Parameters have been described in @cite Senst2012, @cite Senst2013, @cite Senst2014, @cite Senst2016.
    For the RLOF configuration see optflow::RLOFOpticalFlowParameter for further details.
    @note If the grid size is set to (1,1) and the forward backward threshold <= 0 that the dense optical flow field is purely
    computed with the RLOF.

    @note SIMD parallelization is only available when compiling with SSE4.1.
    @note Note that in output, if no correspondences are found between \a I0 and \a I1, the \a flow is set to 0.

    @sa optflow::DenseRLOFOpticalFlow, optflow::RLOFOpticalFlowParameter
    """

@overload
def calcOpticalFlowSF(from_, to, layers, averaging_block_size, max_flow, flow=...) -> flow:
    """
    @overload
    """

@overload
def calcOpticalFlowSF(from_, to, layers, averaging_block_size, max_flow, flow=...) -> flow:
    """
    @brief Calculate an optical flow using "SimpleFlow" algorithm.

    @param from First 8-bit 3-channel image.
    @param to Second 8-bit 3-channel image of the same size as prev
    @param flow computed flow image that has the same size as prev and type CV_32FC2
    @param layers Number of layers
    @param averaging_block_size Size of block through which we sum up when calculate cost function
    for pixel
    @param max_flow maximal flow that we search at each level
    @param sigma_dist vector smooth spatial sigma parameter
    @param sigma_color vector smooth color sigma parameter
    @param postprocess_window window size for postprocess cross bilateral filter
    @param sigma_dist_fix spatial sigma for postprocess cross bilateralf filter
    @param sigma_color_fix color sigma for postprocess cross bilateral filter
    @param occ_thr threshold for detecting occlusions
    @param upscale_averaging_radius window size for bilateral upscale operation
    @param upscale_sigma_dist spatial sigma for bilateral upscale operation
    @param upscale_sigma_color color sigma for bilateral upscale operation
    @param speed_up_thr threshold to detect point with irregular flow - where flow should be
    recalculated after upscale

    See @cite Tao2012 . And site of project - <http://graphics.berkeley.edu/papers/Tao-SAN-2012-05/>.

    @note
       -   An example using the simpleFlow algorithm can be found at samples/simpleflow_demo.cpp
    """

def calcOpticalFlowSparseRLOF(prevImg, nextImg, prevPts, nextPts, status=..., err=..., rlofParam=..., forwardBackwardThreshold=...) -> tuple[nextPts, status, err]:
    """
    @brief Calculates fast optical flow for a sparse feature set using the robust local optical flow (RLOF) similar
    * to optflow::calcOpticalFlowPyrLK().
    *
    * The RLOF is a fast local optical flow approach described in @cite Senst2012 @cite Senst2013 @cite Senst2014
    and @cite Senst2016 similar to the pyramidal iterative Lucas-Kanade method as
    * proposed by @cite Bouguet00. More details and experiments can be found in the following thesis @cite Senst2019.
    * The implementation is derived from optflow::calcOpticalFlowPyrLK().
    *
    * @param prevImg first 8-bit input image. If The cross-based RLOF is used (by selecting optflow::RLOFOpticalFlowParameter::supportRegionType
    * = SupportRegionType::SR_CROSS) image has to be a 8-bit 3 channel image.
    * @param nextImg second 8-bit input image. If The cross-based RLOF is used (by selecting optflow::RLOFOpticalFlowParameter::supportRegionType
    * = SupportRegionType::SR_CROSS) image has to be a 8-bit 3 channel image.
    * @param prevPts vector of 2D points for which the flow needs to be found; point coordinates must be single-precision
    * floating-point numbers.
    * @param nextPts output vector of 2D points (with single-precision floating-point coordinates) containing the calculated
    * new positions of input features in the second image; when optflow::RLOFOpticalFlowParameter::useInitialFlow variable is true  the vector must
    * have the same size as in the input and contain the initialization point correspondences.
    * @param status output status vector (of unsigned chars); each element of the vector is set to 1 if the flow for the
    * corresponding features has passed the forward backward check.
    * @param err output vector of errors; each element of the vector is set to the forward backward error for the corresponding feature.
    * @param rlofParam see optflow::RLOFOpticalFlowParameter
    * @param forwardBackwardThreshold Threshold for the forward backward confidence check. If forewardBackwardThreshold <=0 the forward
    *
    * @note SIMD parallelization is only available when compiling with SSE4.1.
    *
    * Parameters have been described in @cite Senst2012, @cite Senst2013, @cite Senst2014 and @cite Senst2016.
    * For the RLOF configuration see optflow::RLOFOpticalFlowParameter for further details.
    """

def calcOpticalFlowSparseToDense(from_, to, flow=..., grid_step=..., k=..., sigma=..., use_post_proc=..., fgs_lambda=..., fgs_sigma=...) -> flow:
    """
    @brief Fast dense optical flow based on PyrLK sparse matches interpolation.

    @param from first 8-bit 3-channel or 1-channel image.
    @param to  second 8-bit 3-channel or 1-channel image of the same size as from
    @param flow computed flow image that has the same size as from and CV_32FC2 type
    @param grid_step stride used in sparse match computation. Lower values usually
           result in higher quality but slow down the algorithm.
    @param k number of nearest-neighbor matches considered, when fitting a locally affine
           model. Lower values can make the algorithm noticeably faster at the cost of
           some quality degradation.
    @param sigma parameter defining how fast the weights decrease in the locally-weighted affine
           fitting. Higher values can help preserve fine details, lower values can help to get rid
           of the noise in the output flow.
    @param use_post_proc defines whether the ximgproc::fastGlobalSmootherFilter() is used
           for post-processing after interpolation
    @param fgs_lambda see the respective parameter of the ximgproc::fastGlobalSmootherFilter()
    @param fgs_sigma  see the respective parameter of the ximgproc::fastGlobalSmootherFilter()
    """

def createOptFlow_DeepFlow() -> retval:
    """
    @brief DeepFlow optical flow algorithm implementation.

    The class implements the DeepFlow optical flow algorithm described in @cite Weinzaepfel2013 . See
    also <http://lear.inrialpes.fr/src/deepmatching/> .
    Parameters - class fields - that may be modified after creating a class instance:
    -   member float alpha
    Smoothness assumption weight
    -   member float delta
    Color constancy assumption weight
    -   member float gamma
    Gradient constancy weight
    -   member float sigma
    Gaussian smoothing parameter
    -   member int minSize
    Minimal dimension of an image in the pyramid (next, smaller images in the pyramid are generated
    until one of the dimensions reaches this size)
    -   member float downscaleFactor
    Scaling factor in the image pyramid (must be \< 1)
    -   member int fixedPointIterations
    How many iterations on each level of the pyramid
    -   member int sorIterations
    Iterations of Succesive Over-Relaxation (solver)
    -   member float omega
    Relaxation factor in SOR
    """

def createOptFlow_DenseRLOF() -> retval:
    """
    .
    """

def createOptFlow_DualTVL1() -> retval:
    """
    @brief Creates instance of cv::DenseOpticalFlow
    """

def createOptFlow_Farneback() -> retval:
    """
    .
    """

def createOptFlow_PCAFlow() -> retval:
    """
    @brief Creates an instance of PCAFlow
    """

def createOptFlow_SimpleFlow() -> retval:
    """
    .
    """

def createOptFlow_SparseRLOF() -> retval:
    """
    .
    """

def createOptFlow_SparseToDense() -> retval:
    """
    .
    """

GPC_DESCRIPTOR_DCT: Final[int]
GPC_DESCRIPTOR_WHT: Final[int]
INTERP_EPIC: Final[int]
INTERP_GEO: Final[int]
INTERP_RIC: Final[int]
SR_CROSS: Final[int]
SR_FIXED: Final[int]
ST_BILINEAR: Final[int]
ST_STANDART: Final[int]



from typing import Any, TypeAlias

retval: TypeAlias = Any

def getIppVersion() -> retval:
    """
    .
    """

def setUseIPP(flag) -> None:
    """
    .
    """

def setUseIPP_NotExact(flag) -> None:
    """
    .
    """

def useIPP() -> retval:
    """
    proxy for hal::Cholesky
    """

def useIPP_NotExact() -> retval:
    """
    .
    """



import builtins
from typing import Any, Final, overload, TypeAlias

from .. import functions as cv2

depth: TypeAlias = Any
mask: TypeAlias = Any
normals: TypeAlias = Any
out: TypeAlias = Any
plane_coefficients: TypeAlias = Any
Rt: TypeAlias = Any
warpedMask: TypeAlias = Any
points3d: TypeAlias = Any
warpedImage: TypeAlias = Any
registeredDepth: TypeAlias = Any
warpedDepth: TypeAlias = Any
dst: TypeAlias = Any
retval: TypeAlias = Any

class DepthCleaner(cv2.Algorithm):
    def apply(self, points, depth=...) -> depth:
        """
        Given a set of 3d points in a depth image, compute the normals at each point.
        * @param points a rows x cols x 3 matrix of CV_32F/CV64F or a rows x cols x 1 CV_U16S
        * @param depth a rows x cols matrix of the cleaned up depth
        """

    def getDepth(self) -> retval:
        """"""

    def getMethod(self) -> retval:
        """"""

    def getWindowSize(self) -> retval:
        """"""

    def initialize(self) -> None:
        """
        Initializes some data that is cached for later computation
        * If that function is not called, it will be called the first time normals are computed
        """

    def setDepth(self, val) -> None:
        """"""

    def setMethod(self, val) -> None:
        """"""

    def setWindowSize(self, val) -> None:
        """"""

    def create(self, depth, window_size=..., method=...) -> retval:
        """
        Constructor
        * @param depth the depth of the normals (only CV_32F or CV_64F)
        * @param window_size the window size to compute the normals: can only be 1,3,5 or 7
        * @param method one of the methods to use: RGBD_NORMALS_METHOD_SRI, RGBD_NORMALS_METHOD_FALS
        """

class FastICPOdometry(Odometry):
    def getAngleThreshold(self) -> retval:
        """"""

    def getCameraMatrix(self) -> retval:
        """"""

    def getIterationCounts(self) -> retval:
        """"""

    def getKernelSize(self) -> retval:
        """"""

    def getMaxDistDiff(self) -> retval:
        """"""

    def getSigmaDepth(self) -> retval:
        """"""

    def getSigmaSpatial(self) -> retval:
        """"""

    def getTransformType(self) -> retval:
        """"""

    def prepareFrameCache(self, frame, cacheType) -> retval:
        """"""

    def setAngleThreshold(self, f) -> None:
        """"""

    def setCameraMatrix(self, val) -> None:
        """"""

    def setIterationCounts(self, val) -> None:
        """"""

    def setKernelSize(self, f) -> None:
        """"""

    def setMaxDistDiff(self, val) -> None:
        """"""

    def setSigmaDepth(self, f) -> None:
        """"""

    def setSigmaSpatial(self, f) -> None:
        """"""

    def setTransformType(self, val) -> None:
        """"""

    def create(self, cameraMatrix, maxDistDiff=..., angleThreshold=..., sigmaDepth=..., sigmaSpatial=..., kernelSize=..., iterCounts=...) -> retval:
        """
        Constructor.
        * @param cameraMatrix Camera matrix
        * @param maxDistDiff Correspondences between pixels of two given frames will be filtered out *                     if their depth difference is larger than maxDepthDiff
        * @param angleThreshold Correspondence will be filtered out *                     if an angle between their normals is bigger than threshold
        * @param sigmaDepth Depth sigma in meters for bilateral smooth
        * @param sigmaSpatial Spatial sigma in pixels for bilateral smooth
        * @param kernelSize Kernel size in pixels for bilateral smooth
        * @param iterCounts Count of iterations on each pyramid level
        """

class ICPOdometry(Odometry):
    def getCameraMatrix(self) -> retval:
        """"""

    def getIterationCounts(self) -> retval:
        """"""

    def getMaxDepth(self) -> retval:
        """"""

    def getMaxDepthDiff(self) -> retval:
        """"""

    def getMaxPointsPart(self) -> retval:
        """"""

    def getMaxRotation(self) -> retval:
        """"""

    def getMaxTranslation(self) -> retval:
        """"""

    def getMinDepth(self) -> retval:
        """"""

    def getNormalsComputer(self) -> retval:
        """"""

    def getTransformType(self) -> retval:
        """"""

    def prepareFrameCache(self, frame, cacheType) -> retval:
        """"""

    def setCameraMatrix(self, val) -> None:
        """"""

    def setIterationCounts(self, val) -> None:
        """"""

    def setMaxDepth(self, val) -> None:
        """"""

    def setMaxDepthDiff(self, val) -> None:
        """"""

    def setMaxPointsPart(self, val) -> None:
        """"""

    def setMaxRotation(self, val) -> None:
        """"""

    def setMaxTranslation(self, val) -> None:
        """"""

    def setMinDepth(self, val) -> None:
        """"""

    def setTransformType(self, val) -> None:
        """"""

    def create(self, cameraMatrix=..., minDepth=..., maxDepth=..., maxDepthDiff=..., maxPointsPart=..., iterCounts=..., transformType=...) -> retval:
        """
        Constructor.
        * @param cameraMatrix Camera matrix
        * @param minDepth Pixels with depth less than minDepth will not be used
        * @param maxDepth Pixels with depth larger than maxDepth will not be used
        * @param maxDepthDiff Correspondences between pixels of two given frames will be filtered out *                     if their depth difference is larger than maxDepthDiff
        * @param maxPointsPart The method uses a random pixels subset of size frameWidth x frameHeight x pointsPart
        * @param iterCounts Count of iterations on each pyramid level.
        * @param transformType Class of trasformation
        """

class Odometry(cv2.Algorithm):
    def DEFAULT_MAX_DEPTH(self) -> retval:
        """"""

    def DEFAULT_MAX_DEPTH_DIFF(self) -> retval:
        """"""

    def DEFAULT_MAX_POINTS_PART(self) -> retval:
        """"""

    def DEFAULT_MAX_ROTATION(self) -> retval:
        """"""

    def DEFAULT_MAX_TRANSLATION(self) -> retval:
        """"""

    def DEFAULT_MIN_DEPTH(self) -> retval:
        """ """

    def compute(self, srcImage, srcDepth, srcMask, dstImage, dstDepth, dstMask, Rt=..., initRt=...) -> tuple[retval, Rt]:
        """
        Method to compute a transformation from the source frame to the destination one.
        * Some odometry algorithms do not used some data of frames (eg. ICP does not use images).
        * In such case corresponding arguments can be set as empty Mat.
        * The method returns true if all internal computations were possible (e.g. there were enough correspondences,
        * system of equations has a solution, etc) and resulting transformation satisfies some test if it's provided
        * by the Odometry inheritor implementation (e.g. thresholds for maximum translation and rotation).
        * @param srcImage Image data of the source frame (CV_8UC1)
        * @param srcDepth Depth data of the source frame (CV_32FC1, in meters)
        * @param srcMask Mask that sets which pixels have to be used from the source frame (CV_8UC1)
        * @param dstImage Image data of the destination frame (CV_8UC1)
        * @param dstDepth Depth data of the destination frame (CV_32FC1, in meters)
        * @param dstMask Mask that sets which pixels have to be used from the destination frame (CV_8UC1)
        * @param Rt Resulting transformation from the source frame to the destination one (rigid body motion): dst_p = Rt * src_p, where dst_p is a homogeneous point in the destination frame and src_p is homogeneous point in the source frame, Rt is 4x4 matrix of CV_64FC1 type.
        * @param initRt Initial transformation from the source frame to the destination one (optional)
        """

    def compute2(self, srcFrame, dstFrame, Rt=..., initRt=...) -> tuple[retval, Rt]:
        """
        One more method to compute a transformation from the source frame to the destination one.
        * It is designed to save on computing the frame data (image pyramids, normals, etc.).
        """

    def getCameraMatrix(self) -> retval:
        """
        @see setCameraMatrix
        """

    def getTransformType(self) -> retval:
        """
        @see setTransformType
        """

    def prepareFrameCache(self, frame, cacheType) -> retval:
        """
        Prepare a cache for the frame. The function checks the precomputed/passed data (throws the error if this data
        * does not satisfy) and computes all remaining cache data needed for the frame. Returned size is a resolution
        * of the prepared frame.
        * @param frame The odometry which will process the frame.
        * @param cacheType The cache type: CACHE_SRC, CACHE_DST or CACHE_ALL.
        """

    def setCameraMatrix(self, val) -> None:
        """
        @copybrief getCameraMatrix @see getCameraMatrix
        """

    def setTransformType(self, val) -> None:
        """
        @copybrief getTransformType @see getTransformType
        """

    def create(self, odometryType) -> retval:
        """"""

class OdometryFrame(RgbdFrame):
    def release(self) -> None:
        """"""

    def releasePyramids(self) -> None:
        """"""

    def create(self, image=..., depth=..., mask=..., normals=..., ID=...) -> retval:
        """"""

class RgbdFrame:
    def release(self) -> None:
        """"""

    def create(self, image=..., depth=..., mask=..., normals=..., ID=...) -> retval:
        """"""

class RgbdICPOdometry(Odometry):
    def getCameraMatrix(self) -> retval:
        """"""

    def getIterationCounts(self) -> retval:
        """"""

    def getMaxDepth(self) -> retval:
        """"""

    def getMaxDepthDiff(self) -> retval:
        """"""

    def getMaxPointsPart(self) -> retval:
        """"""

    def getMaxRotation(self) -> retval:
        """"""

    def getMaxTranslation(self) -> retval:
        """"""

    def getMinDepth(self) -> retval:
        """"""

    def getMinGradientMagnitudes(self) -> retval:
        """"""

    def getNormalsComputer(self) -> retval:
        """"""

    def getTransformType(self) -> retval:
        """"""

    def prepareFrameCache(self, frame, cacheType) -> retval:
        """"""

    def setCameraMatrix(self, val) -> None:
        """"""

    def setIterationCounts(self, val) -> None:
        """"""

    def setMaxDepth(self, val) -> None:
        """"""

    def setMaxDepthDiff(self, val) -> None:
        """"""

    def setMaxPointsPart(self, val) -> None:
        """"""

    def setMaxRotation(self, val) -> None:
        """"""

    def setMaxTranslation(self, val) -> None:
        """"""

    def setMinDepth(self, val) -> None:
        """"""

    def setMinGradientMagnitudes(self, val) -> None:
        """"""

    def setTransformType(self, val) -> None:
        """"""

    def create(self, cameraMatrix=..., minDepth=..., maxDepth=..., maxDepthDiff=..., maxPointsPart=..., iterCounts=..., minGradientMagnitudes=..., transformType=...) -> retval:
        """
        Constructor.
        * @param cameraMatrix Camera matrix
        * @param minDepth Pixels with depth less than minDepth will not be used
        * @param maxDepth Pixels with depth larger than maxDepth will not be used
        * @param maxDepthDiff Correspondences between pixels of two given frames will be filtered out *                     if their depth difference is larger than maxDepthDiff
        * @param maxPointsPart The method uses a random pixels subset of size frameWidth x frameHeight x pointsPart
        * @param iterCounts Count of iterations on each pyramid level.
        * @param minGradientMagnitudes For each pyramid level the pixels will be filtered out *                              if they have gradient magnitude less than minGradientMagnitudes[level].
        * @param transformType Class of trasformation
        """

class RgbdNormals(cv2.Algorithm):
    def apply(self, points, normals=...) -> normals:
        """
        Given a set of 3d points in a depth image, compute the normals at each point.
        * @param points a rows x cols x 3 matrix of CV_32F/CV64F or a rows x cols x 1 CV_U16S
        * @param normals a rows x cols x 3 matrix
        """

    def getCols(self) -> retval:
        """"""

    def getDepth(self) -> retval:
        """"""

    def getK(self) -> retval:
        """"""

    def getMethod(self) -> retval:
        """"""

    def getRows(self) -> retval:
        """"""

    def getWindowSize(self) -> retval:
        """"""

    def initialize(self) -> None:
        """
        Initializes some data that is cached for later computation
        * If that function is not called, it will be called the first time normals are computed
        """

    def setCols(self, val) -> None:
        """"""

    def setDepth(self, val) -> None:
        """"""

    def setK(self, val) -> None:
        """"""

    def setMethod(self, val) -> None:
        """"""

    def setRows(self, val) -> None:
        """"""

    def setWindowSize(self, val) -> None:
        """"""

    def create(self, rows, cols, depth, K, window_size=..., method=...) -> retval:
        """
        Constructor
        * @param rows the number of rows of the depth image normals will be computed on
        * @param cols the number of cols of the depth image normals will be computed on
        * @param depth the depth of the normals (only CV_32F or CV_64F)
        * @param K the calibration matrix to use
        * @param window_size the window size to compute the normals: can only be 1,3,5 or 7
        * @param method one of the methods to use: RGBD_NORMALS_METHOD_SRI, RGBD_NORMALS_METHOD_FALS
        """

class RgbdOdometry(Odometry):
    def getCameraMatrix(self) -> retval:
        """"""

    def getIterationCounts(self) -> retval:
        """"""

    def getMaxDepth(self) -> retval:
        """"""

    def getMaxDepthDiff(self) -> retval:
        """"""

    def getMaxPointsPart(self) -> retval:
        """"""

    def getMaxRotation(self) -> retval:
        """"""

    def getMaxTranslation(self) -> retval:
        """"""

    def getMinDepth(self) -> retval:
        """"""

    def getMinGradientMagnitudes(self) -> retval:
        """"""

    def getTransformType(self) -> retval:
        """"""

    def prepareFrameCache(self, frame, cacheType) -> retval:
        """"""

    def setCameraMatrix(self, val) -> None:
        """"""

    def setIterationCounts(self, val) -> None:
        """"""

    def setMaxDepth(self, val) -> None:
        """"""

    def setMaxDepthDiff(self, val) -> None:
        """"""

    def setMaxPointsPart(self, val) -> None:
        """"""

    def setMaxRotation(self, val) -> None:
        """"""

    def setMaxTranslation(self, val) -> None:
        """"""

    def setMinDepth(self, val) -> None:
        """"""

    def setMinGradientMagnitudes(self, val) -> None:
        """"""

    def setTransformType(self, val) -> None:
        """"""

    def create(self, cameraMatrix=..., minDepth=..., maxDepth=..., maxDepthDiff=..., iterCounts=..., minGradientMagnitudes=..., maxPointsPart=..., transformType=...) -> retval:
        """
        Constructor.
        * @param cameraMatrix Camera matrix
        * @param minDepth Pixels with depth less than minDepth will not be used (in meters)
        * @param maxDepth Pixels with depth larger than maxDepth will not be used (in meters)
        * @param maxDepthDiff Correspondences between pixels of two given frames will be filtered out *                     if their depth difference is larger than maxDepthDiff (in meters)
        * @param iterCounts Count of iterations on each pyramid level.
        * @param minGradientMagnitudes For each pyramid level the pixels will be filtered out *                              if they have gradient magnitude less than minGradientMagnitudes[level].
        * @param maxPointsPart The method uses a random pixels subset of size frameWidth x frameHeight x pointsPart
        * @param transformType Class of transformation
        """

class RgbdPlane(cv2.Algorithm):
    @overload
    def apply(self, points3d, normals, mask=..., plane_coefficients=...) -> tuple[mask, plane_coefficients]:
        """
        Find The planes in a depth image
        * @param points3d the 3d points organized like the depth image: rows x cols with 3 channels
        * @param normals the normals for every point in the depth image
        * @param mask An image where each pixel is labeled with the plane it belongs to *        and 255 if it does not belong to any plane
        * @param plane_coefficients the coefficients of the corresponding planes (a,b,c,d) such that ax+by+cz+d=0, norm(a,b,c)=1 *        and c < 0 (so that the normal points towards the camera)
        """

    @overload
    def apply(self, points3d, mask=..., plane_coefficients=...) -> tuple[mask, plane_coefficients]:
        """
        Find The planes in a depth image but without doing a normal check, which is faster but less accurate
        * @param points3d the 3d points organized like the depth image: rows x cols with 3 channels
        * @param mask An image where each pixel is labeled with the plane it belongs to *        and 255 if it does not belong to any plane
        * @param plane_coefficients the coefficients of the corresponding planes (a,b,c,d) such that ax+by+cz+d=0
        """

    def getBlockSize(self) -> retval:
        """"""

    def getMethod(self) -> retval:
        """"""

    def getMinSize(self) -> retval:
        """"""

    def getSensorErrorA(self) -> retval:
        """"""

    def getSensorErrorB(self) -> retval:
        """"""

    def getSensorErrorC(self) -> retval:
        """"""

    def getThreshold(self) -> retval:
        """"""

    def setBlockSize(self, val) -> None:
        """"""

    def setMethod(self, val) -> None:
        """"""

    def setMinSize(self, val) -> None:
        """"""

    def setSensorErrorA(self, val) -> None:
        """"""

    def setSensorErrorB(self, val) -> None:
        """"""

    def setSensorErrorC(self, val) -> None:
        """"""

    def setThreshold(self, val) -> None:
        """"""

    def create(self, method, block_size, min_size, threshold, sensor_error_a=..., sensor_error_b=..., sensor_error_c=...) -> retval:
        """
        Constructor
        * @param block_size The size of the blocks to look at for a stable MSE
        * @param min_size The minimum size of a cluster to be considered a plane
        * @param threshold The maximum distance of a point from a plane to belong to it (in meters)
        * @param sensor_error_a coefficient of the sensor error. 0 by default, 0.0075 for a Kinect
        * @param sensor_error_b coefficient of the sensor error. 0 by default
        * @param sensor_error_c coefficient of the sensor error. 0 by default
        * @param method The method to use to compute the planes.
        """

def DepthCleaner_create(depth, window_size=..., method=...) -> retval:
    """
    Constructor
         * @param depth the depth of the normals (only CV_32F or CV_64F)
         * @param window_size the window size to compute the normals: can only be 1,3,5 or 7
         * @param method one of the methods to use: RGBD_NORMALS_METHOD_SRI, RGBD_NORMALS_METHOD_FALS
    """

def FastICPOdometry_create(cameraMatrix, maxDistDiff=..., angleThreshold=..., sigmaDepth=..., sigmaSpatial=..., kernelSize=..., iterCounts=...) -> retval:
    """
    Constructor.
         * @param cameraMatrix Camera matrix
         * @param maxDistDiff Correspondences between pixels of two given frames will be filtered out
         *                     if their depth difference is larger than maxDepthDiff
         * @param angleThreshold Correspondence will be filtered out
         *                     if an angle between their normals is bigger than threshold
         * @param sigmaDepth Depth sigma in meters for bilateral smooth
         * @param sigmaSpatial Spatial sigma in pixels for bilateral smooth
         * @param kernelSize Kernel size in pixels for bilateral smooth
         * @param iterCounts Count of iterations on each pyramid level
    """

def ICPOdometry_create(cameraMatrix=..., minDepth=..., maxDepth=..., maxDepthDiff=..., maxPointsPart=..., iterCounts=..., transformType=...) -> retval:
    """
    Constructor.
         * @param cameraMatrix Camera matrix
         * @param minDepth Pixels with depth less than minDepth will not be used
         * @param maxDepth Pixels with depth larger than maxDepth will not be used
         * @param maxDepthDiff Correspondences between pixels of two given frames will be filtered out
         *                     if their depth difference is larger than maxDepthDiff
         * @param maxPointsPart The method uses a random pixels subset of size frameWidth x frameHeight x pointsPart
         * @param iterCounts Count of iterations on each pyramid level.
         * @param transformType Class of trasformation
    """

def OdometryFrame_create(image=..., depth=..., mask=..., normals=..., ID=...) -> retval:
    """
    .
    """

def Odometry_create(odometryType) -> retval:
    """
    .
    """

def RgbdFrame_create(image=..., depth=..., mask=..., normals=..., ID=...) -> retval:
    """
    .
    """

def RgbdICPOdometry_create(cameraMatrix=..., minDepth=..., maxDepth=..., maxDepthDiff=..., maxPointsPart=..., iterCounts=..., minGradientMagnitudes=..., transformType=...) -> retval:
    """
    Constructor.
         * @param cameraMatrix Camera matrix
         * @param minDepth Pixels with depth less than minDepth will not be used
         * @param maxDepth Pixels with depth larger than maxDepth will not be used
         * @param maxDepthDiff Correspondences between pixels of two given frames will be filtered out
         *                     if their depth difference is larger than maxDepthDiff
         * @param maxPointsPart The method uses a random pixels subset of size frameWidth x frameHeight x pointsPart
         * @param iterCounts Count of iterations on each pyramid level.
         * @param minGradientMagnitudes For each pyramid level the pixels will be filtered out
         *                              if they have gradient magnitude less than minGradientMagnitudes[level].
         * @param transformType Class of trasformation
    """

def RgbdNormals_create(rows, cols, depth, K, window_size=..., method=...) -> retval:
    """
    Constructor
         * @param rows the number of rows of the depth image normals will be computed on
         * @param cols the number of cols of the depth image normals will be computed on
         * @param depth the depth of the normals (only CV_32F or CV_64F)
         * @param K the calibration matrix to use
         * @param window_size the window size to compute the normals: can only be 1,3,5 or 7
         * @param method one of the methods to use: RGBD_NORMALS_METHOD_SRI, RGBD_NORMALS_METHOD_FALS
    """

def RgbdOdometry_create(cameraMatrix=..., minDepth=..., maxDepth=..., maxDepthDiff=..., iterCounts=..., minGradientMagnitudes=..., maxPointsPart=..., transformType=...) -> retval:
    """
    Constructor.
         * @param cameraMatrix Camera matrix
         * @param minDepth Pixels with depth less than minDepth will not be used (in meters)
         * @param maxDepth Pixels with depth larger than maxDepth will not be used (in meters)
         * @param maxDepthDiff Correspondences between pixels of two given frames will be filtered out
         *                     if their depth difference is larger than maxDepthDiff (in meters)
         * @param iterCounts Count of iterations on each pyramid level.
         * @param minGradientMagnitudes For each pyramid level the pixels will be filtered out
         *                              if they have gradient magnitude less than minGradientMagnitudes[level].
         * @param maxPointsPart The method uses a random pixels subset of size frameWidth x frameHeight x pointsPart
         * @param transformType Class of transformation
    """

def RgbdPlane_create(method, block_size, min_size, threshold, sensor_error_a=..., sensor_error_b=..., sensor_error_c=...) -> retval:
    """
    Constructor
         * @param block_size The size of the blocks to look at for a stable MSE
         * @param min_size The minimum size of a cluster to be considered a plane
         * @param threshold The maximum distance of a point from a plane to belong to it (in meters)
         * @param sensor_error_a coefficient of the sensor error. 0 by default, 0.0075 for a Kinect
         * @param sensor_error_b coefficient of the sensor error. 0 by default
         * @param sensor_error_c coefficient of the sensor error. 0 by default
         * @param method The method to use to compute the planes.
    """

def depthTo3d(depth, K, points3d=..., mask=...) -> points3d:
    """
    Converts a depth image to an organized set of 3d points.
       * The coordinate system is x pointing left, y down and z away from the camera
       * @param depth the depth image (if given as short int CV_U, it is assumed to be the depth in millimeters
       *              (as done with the Microsoft Kinect), otherwise, if given as CV_32F or CV_64F, it is assumed in meters)
       * @param K The calibration matrix
       * @param points3d the resulting 3d points. They are of depth the same as `depth` if it is CV_32F or CV_64F, and the
       *        depth of `K` if `depth` is of depth CV_U
       * @param mask the mask of the points to consider (can be empty)
    """

def depthTo3dSparse(depth, in_K, in_points, points3d=...) -> points3d:
    """
    * @param depth the depth image
       * @param in_K
       * @param in_points the list of xy coordinates
       * @param points3d the resulting 3d points
    """

def registerDepth(unregisteredCameraMatrix, registeredCameraMatrix, registeredDistCoeffs, Rt, unregisteredDepth, outputImagePlaneSize, registeredDepth=..., depthDilation=...) -> registeredDepth:
    """
    Registers depth data to an external camera
       * Registration is performed by creating a depth cloud, transforming the cloud by
       * the rigid body transformation between the cameras, and then projecting the
       * transformed points into the RGB camera.
       *
       * uv_rgb = K_rgb * [R | t] * z * inv(K_ir) * uv_ir
       *
       * Currently does not check for negative depth values.
       *
       * @param unregisteredCameraMatrix the camera matrix of the depth camera
       * @param registeredCameraMatrix the camera matrix of the external camera
       * @param registeredDistCoeffs the distortion coefficients of the external camera
       * @param Rt the rigid body transform between the cameras. Transforms points from depth camera frame to external camera frame.
       * @param unregisteredDepth the input depth data
       * @param outputImagePlaneSize the image plane dimensions of the external camera (width, height)
       * @param registeredDepth the result of transforming the depth into the external camera
       * @param depthDilation whether or not the depth is dilated to avoid holes and occlusion errors (optional)
    """

def rescaleDepth(in_, depth, out=..., depth_factor=...) -> out:
    """
    If the input image is of type CV_16UC1 (like the Kinect one), the image is converted to floats, divided
       * by depth_factor to get a depth in meters, and the values 0 are converted to std::numeric_limits<float>::quiet_NaN()
       * Otherwise, the image is simply converted to floats
       * @param in the depth image (if given as short int CV_U, it is assumed to be the depth in millimeters
       *              (as done with the Microsoft Kinect), it is assumed in meters)
       * @param depth the desired output depth (floats or double)
       * @param out The rescaled float depth image
       * @param depth_factor (optional) factor by which depth is converted to distance (by default = 1000.0 for Kinect sensor)
    """

def warpFrame(image, depth, mask, Rt, cameraMatrix, distCoeff, warpedImage=..., warpedDepth=..., warpedMask=...) -> tuple[warpedImage, warpedDepth, warpedMask]:
    """
    Warp the image: compute 3d points from the depth, transform them using given transformation,
       * then project color point cloud to an image plane.
       * This function can be used to visualize results of the Odometry algorithm.
       * @param image The image (of CV_8UC1 or CV_8UC3 type)
       * @param depth The depth (of type used in depthTo3d fuction)
       * @param mask The mask of used pixels (of CV_8UC1), it can be empty
       * @param Rt The transformation that will be applied to the 3d points computed from the depth
       * @param cameraMatrix Camera matrix
       * @param distCoeff Distortion coefficients
       * @param warpedImage The warped image.
       * @param warpedDepth The warped depth.
       * @param warpedMask The warped mask.
    """

DEPTH_CLEANER_DEPTH_CLEANER_NIL: Final[int]
DepthCleaner_DEPTH_CLEANER_NIL: Final[int]
ODOMETRY_FRAME_CACHE_ALL: Final[int]
ODOMETRY_FRAME_CACHE_DST: Final[int]
ODOMETRY_FRAME_CACHE_SRC: Final[int]
ODOMETRY_RIGID_BODY_MOTION: Final[int]
ODOMETRY_ROTATION: Final[int]
ODOMETRY_TRANSLATION: Final[int]
OdometryFrame_CACHE_ALL: Final[int]
OdometryFrame_CACHE_DST: Final[int]
OdometryFrame_CACHE_SRC: Final[int]
Odometry_RIGID_BODY_MOTION: Final[int]
Odometry_ROTATION: Final[int]
Odometry_TRANSLATION: Final[int]
RGBD_NORMALS_RGBD_NORMALS_METHOD_FALS: Final[int]
RGBD_NORMALS_RGBD_NORMALS_METHOD_LINEMOD: Final[int]
RGBD_NORMALS_RGBD_NORMALS_METHOD_SRI: Final[int]
RGBD_PLANE_RGBD_PLANE_METHOD_DEFAULT: Final[int]
RgbdNormals_RGBD_NORMALS_METHOD_FALS: Final[int]
RgbdNormals_RGBD_NORMALS_METHOD_LINEMOD: Final[int]
RgbdNormals_RGBD_NORMALS_METHOD_SRI: Final[int]
RgbdPlane_RGBD_PLANE_METHOD_DEFAULT: Final[int]



import builtins
from typing import Any, overload, TypeAlias

from .. import functions as cv2

matches: TypeAlias = Any
outImage: TypeAlias = Any
descriptors: TypeAlias = Any
keylines: TypeAlias = Any
keypoints: TypeAlias = Any
outImg: TypeAlias = Any

retval: TypeAlias = Any

class BinaryDescriptor(cv2.Algorithm):
    def compute(self, image, keylines, descriptors=..., returnFloatDescr=...) -> tuple[keylines, descriptors]:
        """
        @brief Requires descriptors computation

        @param image input image
        @param keylines vector containing lines for which descriptors must be computed
        @param descriptors
        @param returnFloatDescr flag (when set to true, original non-binary descriptors are returned)
        """

    def detect(self, image, mask=...) -> keypoints:
        """
        @brief Requires line detection

        @param image input image
        @param keypoints vector that will store extracted lines for one or more images
        @param mask mask matrix to detect only KeyLines of interest
        """

    def getNumOfOctaves(self) -> retval:
        """
        @brief Get current number of octaves
        """

    def getReductionRatio(self) -> retval:
        """
        @brief Get current reduction ratio (used in Gaussian pyramids)
        """

    def getWidthOfBand(self) -> retval:
        """
        @brief Get current width of bands
        """

    def setNumOfOctaves(self, octaves) -> None:
        """
        @brief Set number of octaves
        @param octaves number of octaves
        """

    def setReductionRatio(self, rRatio) -> None:
        """
        @brief Set reduction ratio (used in Gaussian pyramids)
        @param rRatio reduction ratio
        """

    def setWidthOfBand(self, width) -> None:
        """
        @brief Set width of bands
        @param width width of bands
        """

    def createBinaryDescriptor(self) -> retval:
        """
        @brief Create a BinaryDescriptor object with default parameters (or with the ones provided)
        and return a smart pointer to it
        """

class BinaryDescriptorMatcher(cv2.Algorithm):
    def knnMatch(self, queryDescriptors, trainDescriptors, k, mask=..., compactResult=...) -> matches:
        """
        @brief For every input query descriptor, retrieve the best *k* matching ones from a dataset provided from
        user or from the one internal to class

        @param queryDescriptors query descriptors
        @param trainDescriptors dataset of descriptors furnished by user
        @param matches vector to host retrieved matches
        @param k number of the closest descriptors to be returned for every input query
        @param mask mask to select which input descriptors must be matched to ones in dataset
        @param compactResult flag to obtain a compact result (if true, a vector that doesn't contain any matches for a given query is not inserted in final result)
        """

    def knnMatchQuery(self, queryDescriptors, matches, k, masks=..., compactResult=...) -> None:
        """
        @overload
        @param queryDescriptors query descriptors
        @param matches vector to host retrieved matches
        @param k number of the closest descriptors to be returned for every input query
        @param masks vector of masks to select which input descriptors must be matched to ones in dataset (the *i*-th mask in vector indicates whether each input query can be matched with descriptors in dataset relative to *i*-th image)
        @param compactResult flag to obtain a compact result (if true, a vector that doesn't contain any matches for a given query is not inserted in final result)
        """

    def match(self, queryDescriptors, trainDescriptors, mask=...) -> matches:
        """
        @brief For every input query descriptor, retrieve the best matching one from a dataset provided from user
        or from the one internal to class

        @param queryDescriptors query descriptors
        @param trainDescriptors dataset of descriptors furnished by user
        @param matches vector to host retrieved matches
        @param mask mask to select which input descriptors must be matched to one in dataset
        """

    def matchQuery(self, queryDescriptors, masks=...) -> matches:
        """
        @overload
        @param queryDescriptors query descriptors
        @param matches vector to host retrieved matches
        @param masks vector of masks to select which input descriptors must be matched to one in dataset (the *i*-th mask in vector indicates whether each input query can be matched with descriptors in dataset relative to *i*-th image)
        """

class DrawLinesMatchesFlags: ...

class KeyLine:
    def getEndPoint(self) -> retval:
        """
        Returns the end point of the line in the original image
        """

    def getEndPointInOctave(self) -> retval:
        """
        Returns the end point of the line in the octave it was extracted from
        """

    def getStartPoint(self) -> retval:
        """
        Returns the start point of the line in the original image
        """

    def getStartPointInOctave(self) -> retval:
        """
        Returns the start point of the line in the octave it was extracted from
        """

class LSDDetector(cv2.Algorithm):
    @overload
    def detect(self, image, scale, numOctaves, mask=...) -> keypoints:
        """
        @brief Detect lines inside an image.

        @param image input image
        @param keypoints vector that will store extracted lines for one or more images
        @param scale scale factor used in pyramids generation
        @param numOctaves number of octaves inside pyramid
        @param mask mask matrix to detect only KeyLines of interest
        """

    @overload
    def detect(self, images, keylines, scale, numOctaves, masks=...) -> None:
        """
        @overload
        @param images input images
        @param keylines set of vectors that will store extracted lines for one or more images
        @param scale scale factor used in pyramids generation
        @param numOctaves number of octaves inside pyramid
        @param masks vector of mask matrices to detect only KeyLines of interest from each input image
        """

    def createLSDDetector(self) -> retval:
        """
        @brief Creates ad LSDDetector object, using smart pointers.
        """

    def createLSDDetectorWithParams(self, params) -> retval:
        """"""

class LSDParam: ...

def BinaryDescriptor_createBinaryDescriptor() -> retval:
    """
    @brief Create a BinaryDescriptor object with default parameters (or with the ones provided)
      and return a smart pointer to it
    """

def LSDDetector_createLSDDetector() -> retval:
    """
    @brief Creates ad LSDDetector object, using smart pointers.
    """

def LSDDetector_createLSDDetectorWithParams(params) -> retval:
    """
    .
    """

def drawKeylines(image, keylines, outImage=..., color=..., flags=...) -> outImage:
    """
    @brief Draws keylines.

    @param image input image
    @param keylines keylines to be drawn
    @param outImage output image to draw on
    @param color color of lines to be drawn (if set to defaul value, color is chosen randomly)
    @param flags drawing flags
    """

def drawLineMatches(img1, keylines1, img2, keylines2, matches1to2, outImg=..., matchColor=..., singleLineColor=..., matchesMask=..., flags=...) -> outImg:
    """
    @brief Draws the found matches of keylines from two images.

    @param img1 first image
    @param keylines1 keylines extracted from first image
    @param img2 second image
    @param keylines2 keylines extracted from second image
    @param matches1to2 vector of matches
    @param outImg output matrix to draw on
    @param matchColor drawing color for matches (chosen randomly in case of default value)
    @param singleLineColor drawing color for keylines (chosen randomly in case of default value)
    @param matchesMask mask to indicate which matches must be drawn
    @param flags drawing flags, see DrawLinesMatchesFlags

    @note If both *matchColor* and *singleLineColor* are set to their default values, function draws
    matched lines and line connecting them with same color
    """



import builtins
from typing import Any, Final, TypeAlias

from .. import functions as cv2

gtMask: TypeAlias = Any
fgmask: TypeAlias = Any
backgroundImage: TypeAlias = Any
frame: TypeAlias = Any
retval: TypeAlias = Any

class BackgroundSubtractorCNT(cv2.BackgroundSubtractor):
    def apply(self, image, fgmask=..., learningRate=...) -> fgmask:
        """"""

    def getBackgroundImage(self, backgroundImage=...) -> backgroundImage:
        """"""

    def getIsParallel(self) -> retval:
        """
        @brief Returns if we're parallelizing the algorithm.
        """

    def getMaxPixelStability(self) -> retval:
        """
        @brief Returns maximum allowed credit for a pixel in history.
        """

    def getMinPixelStability(self) -> retval:
        """
        @brief Returns number of frames with same pixel color to consider stable.
        """

    def getUseHistory(self) -> retval:
        """
        @brief Returns if we're giving a pixel credit for being stable for a long time.
        """

    def setIsParallel(self, value) -> None:
        """
        @brief Sets if we're parallelizing the algorithm.
        """

    def setMaxPixelStability(self, value) -> None:
        """
        @brief Sets the maximum allowed credit for a pixel in history.
        """

    def setMinPixelStability(self, value) -> None:
        """
        @brief Sets the number of frames with same pixel color to consider stable.
        """

    def setUseHistory(self, value) -> None:
        """
        @brief Sets if we're giving a pixel credit for being stable for a long time.
        """

class BackgroundSubtractorGMG(cv2.BackgroundSubtractor):
    def getBackgroundPrior(self) -> retval:
        """
        @brief Returns the prior probability that each individual pixel is a background pixel.
        """

    def getDecisionThreshold(self) -> retval:
        """
        @brief Returns the value of decision threshold.

        Decision value is the value above which pixel is determined to be FG.
        """

    def getDefaultLearningRate(self) -> retval:
        """
        @brief Returns the learning rate of the algorithm.

        It lies between 0.0 and 1.0. It determines how quickly features are "forgotten" from
        histograms.
        """

    def getMaxFeatures(self) -> retval:
        """
        @brief Returns total number of distinct colors to maintain in histogram.
        """

    def getMaxVal(self) -> retval:
        """
        @brief Returns the maximum value taken on by pixels in image sequence. e.g. 1.0 or 255.
        """

    def getMinVal(self) -> retval:
        """
        @brief Returns the minimum value taken on by pixels in image sequence. Usually 0.
        """

    def getNumFrames(self) -> retval:
        """
        @brief Returns the number of frames used to initialize background model.
        """

    def getQuantizationLevels(self) -> retval:
        """
        @brief Returns the parameter used for quantization of color-space.

        It is the number of discrete levels in each channel to be used in histograms.
        """

    def getSmoothingRadius(self) -> retval:
        """
        @brief Returns the kernel radius used for morphological operations
        """

    def getUpdateBackgroundModel(self) -> retval:
        """
        @brief Returns the status of background model update
        """

    def setBackgroundPrior(self, bgprior) -> None:
        """
        @brief Sets the prior probability that each individual pixel is a background pixel.
        """

    def setDecisionThreshold(self, thresh) -> None:
        """
        @brief Sets the value of decision threshold.
        """

    def setDefaultLearningRate(self, lr) -> None:
        """
        @brief Sets the learning rate of the algorithm.
        """

    def setMaxFeatures(self, maxFeatures) -> None:
        """
        @brief Sets total number of distinct colors to maintain in histogram.
        """

    def setMaxVal(self, val) -> None:
        """
        @brief Sets the maximum value taken on by pixels in image sequence.
        """

    def setMinVal(self, val) -> None:
        """
        @brief Sets the minimum value taken on by pixels in image sequence.
        """

    def setNumFrames(self, nframes) -> None:
        """
        @brief Sets the number of frames used to initialize background model.
        """

    def setQuantizationLevels(self, nlevels) -> None:
        """
        @brief Sets the parameter used for quantization of color-space
        """

    def setSmoothingRadius(self, radius) -> None:
        """
        @brief Sets the kernel radius used for morphological operations
        """

    def setUpdateBackgroundModel(self, update) -> None:
        """
        @brief Sets the status of background model update
        """

class BackgroundSubtractorGSOC(cv2.BackgroundSubtractor):
    def apply(self, image, fgmask=..., learningRate=...) -> fgmask:
        """"""

    def getBackgroundImage(self, backgroundImage=...) -> backgroundImage:
        """"""

class BackgroundSubtractorLSBP(cv2.BackgroundSubtractor):
    def apply(self, image, fgmask=..., learningRate=...) -> fgmask:
        """"""

    def getBackgroundImage(self, backgroundImage=...) -> backgroundImage:
        """"""

class BackgroundSubtractorLSBPDesc: ...

class BackgroundSubtractorMOG(cv2.BackgroundSubtractor):
    def getBackgroundRatio(self) -> retval:
        """"""

    def getHistory(self) -> retval:
        """"""

    def getNMixtures(self) -> retval:
        """"""

    def getNoiseSigma(self) -> retval:
        """"""

    def setBackgroundRatio(self, backgroundRatio) -> None:
        """"""

    def setHistory(self, nframes) -> None:
        """"""

    def setNMixtures(self, nmix) -> None:
        """"""

    def setNoiseSigma(self, noiseSigma) -> None:
        """"""

class SyntheticSequenceGenerator(cv2.Algorithm):
    def getNextFrame(self, frame=..., gtMask=...) -> tuple[frame, gtMask]:
        """
        @brief Obtain the next frame in the sequence.

        @param frame Output frame.
        @param gtMask Output ground-truth (reference) segmentation mask object/background.
        """

def createBackgroundSubtractorCNT(minPixelStability=..., useHistory=..., maxPixelStability=..., isParallel=...) -> retval:
    """
    @brief Creates a CNT Background Subtractor

    @param minPixelStability number of frames with same pixel color to consider stable
    @param useHistory determines if we're giving a pixel credit for being stable for a long time
    @param maxPixelStability maximum allowed credit for a pixel in history
    @param isParallel determines if we're parallelizing the algorithm
    """

def createBackgroundSubtractorGMG(initializationFrames=..., decisionThreshold=...) -> retval:
    """
    @brief Creates a GMG Background Subtractor

    @param initializationFrames number of frames used to initialize the background models.
    @param decisionThreshold Threshold value, above which it is marked foreground, else background.
    """

def createBackgroundSubtractorGSOC(mc=..., nSamples=..., replaceRate=..., propagationRate=..., hitsThreshold=..., alpha=..., beta=..., blinkingSupressionDecay=..., blinkingSupressionMultiplier=..., noiseRemovalThresholdFacBG=..., noiseRemovalThresholdFacFG=...) -> retval:
    """
    @brief Creates an instance of BackgroundSubtractorGSOC algorithm.

    Implementation of the different yet better algorithm which is called GSOC, as it was implemented during GSOC and was not originated from any paper.

    @param mc Whether to use camera motion compensation.
    @param nSamples Number of samples to maintain at each point of the frame.
    @param replaceRate Probability of replacing the old sample - how fast the model will update itself.
    @param propagationRate Probability of propagating to neighbors.
    @param hitsThreshold How many positives the sample must get before it will be considered as a possible replacement.
    @param alpha Scale coefficient for threshold.
    @param beta Bias coefficient for threshold.
    @param blinkingSupressionDecay Blinking supression decay factor.
    @param blinkingSupressionMultiplier Blinking supression multiplier.
    @param noiseRemovalThresholdFacBG Strength of the noise removal for background points.
    @param noiseRemovalThresholdFacFG Strength of the noise removal for foreground points.
    """

def createBackgroundSubtractorLSBP(mc=..., nSamples=..., LSBPRadius=..., Tlower=..., Tupper=..., Tinc=..., Tdec=..., Rscale=..., Rincdec=..., noiseRemovalThresholdFacBG=..., noiseRemovalThresholdFacFG=..., LSBPthreshold=..., minCount=...) -> retval:
    """
    @brief Creates an instance of BackgroundSubtractorLSBP algorithm.

    Background Subtraction using Local SVD Binary Pattern. More details about the algorithm can be found at @cite LGuo2016

    @param mc Whether to use camera motion compensation.
    @param nSamples Number of samples to maintain at each point of the frame.
    @param LSBPRadius LSBP descriptor radius.
    @param Tlower Lower bound for T-values. See @cite LGuo2016 for details.
    @param Tupper Upper bound for T-values. See @cite LGuo2016 for details.
    @param Tinc Increase step for T-values. See @cite LGuo2016 for details.
    @param Tdec Decrease step for T-values. See @cite LGuo2016 for details.
    @param Rscale Scale coefficient for threshold values.
    @param Rincdec Increase/Decrease step for threshold values.
    @param noiseRemovalThresholdFacBG Strength of the noise removal for background points.
    @param noiseRemovalThresholdFacFG Strength of the noise removal for foreground points.
    @param LSBPthreshold Threshold for LSBP binary string.
    @param minCount Minimal number of matches for sample to be considered as foreground.
    """

def createBackgroundSubtractorMOG(history=..., nmixtures=..., backgroundRatio=..., noiseSigma=...) -> retval:
    """
    @brief Creates mixture-of-gaussian background subtractor

    @param history Length of the history.
    @param nmixtures Number of Gaussian mixtures.
    @param backgroundRatio Background ratio.
    @param noiseSigma Noise strength (standard deviation of the brightness or each color channel). 0
    means some automatic value.
    """

def createSyntheticSequenceGenerator(background, object, amplitude=..., wavelength=..., wavespeed=..., objspeed=...) -> retval:
    """
    @brief Creates an instance of SyntheticSequenceGenerator.

    @param background Background image for object.
    @param object Object image which will move slowly over the background.
    @param amplitude Amplitude of wave distortion applied to background.
    @param wavelength Length of waves in distortion applied to background.
    @param wavespeed How fast waves will move.
    @param objspeed How fast object will fly over background.
    """

LSBP_CAMERA_MOTION_COMPENSATION_LK: Final[int]
LSBP_CAMERA_MOTION_COMPENSATION_NONE: Final[int]



import argparse
import inspect
from pathlib import Path

import cv2


def main() -> None:
    parser = argparse.ArgumentParser(description="Script to generate the stubs for the opencv classes.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--output-path", "-o", type=Path, default=Path("cv2_constants.pyi"), help="Output path.")
    parser.add_argument("--merge-with", "-m", type=Path, default=None,
                        help="Path to a file with some constants already defined.")
    args = parser.parse_args()

    output_path: Path = args.output_path
    merge_with_path: Path | None = args.merge_with

    existing_constants: set[str] = set()
    if merge_with_path is not None:
        with merge_with_path.open("r", encoding="utf-8") as constants_file:
            for line in constants_file:
                if ": int" in line:
                    existing_constants.add(line.split(":")[0])

    stubs: list[str] = ["\n\n\n"]
    for (name, member) in inspect.getmembers(cv2):
        if (not inspect.isfunction(member)
            and not inspect.isclass(member)
            and not inspect.isbuiltin(member)
            and not inspect.ismodule(member)
            and not name.startswith("_")
            and name not in existing_constants):
            stubs.append(f"{name}: int\n")

    with output_path.open("a", encoding="utf-8") as stub_file:
        stub_file.writelines(stubs)
    print("Finished")


if __name__ == "__main__":
    main()



import re
import subprocess


def process_function_signature(method_signature: str) -> str:
    method_signature = process_default_args(method_signature)
    method_signature = process_tuple_return(method_signature)
    return method_signature


def process_method_signature(method_signature: str) -> str:
    method_signature = process_default_args(method_signature)
    method_signature = process_tuple_return(method_signature)
    method_signature = add_self(method_signature)
    return method_signature


def process_default_args(function_signature: str) -> str:
    """Function to convert OpenCV's way of representing arguments with a default value to python synthax.

    Examples:
        >>>s = "imdecode(buf, flags) -> retval"
        >>>process_default_args(s)
        "imdecode(buf, flags) -> retval"

        >>>s = "compute(images, keypoints[, descriptors]) -> keypoints, descriptors"
        >>>process_default_args(s)
        "compute(images, keypoints, descriptors = ...) -> keypoints, descriptors"

        >>>s = "imreadmulti(filename[, mats[, flags]]) -> retval, mats"
        >>>process_default_args(s)
        "imreadmulti(filename, mats = ..., flags = ...) -> retval, mats""
    """
    parts = function_signature.split("[")
    for i in range(1, len(parts)-1):
        parts[i] = parts[i] + " = ..."
    if len(parts) > 1:
        parts[-1] = re.sub("]+", " = ...", parts[-1])

    signature = "".join(parts)
    signature = re.sub(r"\(, ", "(", signature)  # If the function had only arguments with default values.
    return signature


def process_tuple_return(function_signature: str) -> str:
    """Function to convert OpenCV's way of representing multiple return values to python synthax.

    Examples:
        >>>s = "computeBitmaps(img[, tb[, eb]]) -> tb, eb"
        >>>process_tuple_return(s)
        "computeBitmaps(img[, tb[, eb]]) -> tuple[tb, eb]"
    """
    parts = function_signature.split(" -> ")
    return_values = parts[-1].split(", ")
    if len(return_values) > 1:
        parts[-1] = f"tuple[{parts[-1]}]"

    return " -> ".join(parts)


def add_self(method_signature: str) -> str:
    """Function to add self as the first argument of a method.

    Examples:
        >>>s = "writeComment(comment, append = ...) -> None:"
        >>>add_self(s)
        "writeComment(self, comment, append = ...) -> None:"

        >>>s = "getIntParam() -> retval:"
        >>>add_self(s)
        "getIntParam(self) -> retval:"
    """
    if len(method_signature) == 0:
        return "DELETE ME!"
    parts = method_signature.split("(", maxsplit=1)
    if parts[-1][0] != ")":
        parts[-1] = "self, " + parts[-1]
    else:
        parts[-1] = "self" + parts[-1]
    return "(".join(parts)


def process_class(name: str, stubs: list[str]) -> None:
    """Adds the methods of `name` to the stubs."""
    print(f"    Adding class: {name}")
    result = subprocess.run(["python", "-m", "pydoc", f"cv2.{name}"], stdout=subprocess.PIPE)
    help_text = result.stdout.split(b"\n\n", maxsplit=1)[1].decode()
    help_text_lines = help_text.splitlines()

    class_path, class_signature = help_text_lines[0].split(" = ")

    # Not in root module
    if len(class_path[4:].split("_")) != 1 and "." not in class_path[4:]:
        stubs.append("")
        stubs.append(f"{class_path[4:]} = {class_path[4:].split('_')[0]}.{class_signature.split(' ')[1].split('(')[0]}")
        stubs.append("")
        return

    stubs.append("")  # New line
    stubs.append(class_signature + ":")  # class definition

    line_idx = 1
    found_at_least_one_method = False
    while line_idx < len(help_text_lines):
        if "(...)" in help_text_lines[line_idx]:
            found_at_least_one_method = True
            # Add function signature.
            line_idx += 1
            stubs.append(f"    def {process_method_signature(help_text_lines[line_idx].replace(' | ', '', 1))}:")
            line_idx += 1
            # Add docstring.
            stubs.append('        """')
            while (line_idx < len(help_text_lines)
                    and "(...)" not in help_text_lines[line_idx]
                    and " |  --" not in help_text_lines[line_idx]):
                if "." in help_text_lines[line_idx]:
                    line = help_text_lines[line_idx].split(".", maxsplit=1)[1].lstrip()
                    if line == "@overload":
                        stubs.insert(-2, 4*" " + line)
                    if line == "* @overload":
                        stubs.insert(-2, 4*" " + "@overload")
                    elif "@param" in stubs[-1] and "@param" not in line:
                        stubs[-1] = stubs[-1] + " " + line  # pyright: ignore
                    else:
                        stubs.append(8*" " + line)
                elif re.match(r" \|      [a-zA-Z]", help_text_lines[line_idx][:9]):
                    stubs.append('        """')
                    stubs.append("")  # New line
                    stubs.append(f"    def {process_method_signature(help_text_lines[line_idx][8:])}:")
                    stubs.append('        """')
                line_idx += 1
            stubs.append('        """')
            stubs.append("")  # New line
        else:
            line_idx += 1

        if line_idx >= len(help_text_lines) or "inherited" in help_text_lines[line_idx]:
            break
    if not found_at_least_one_method:
        stubs.append("    ...")
        stubs.append("")  # New line


def process_function(name: str, stubs: list[str]) -> None:
    print(f"    Adding function: {name}")
    result = subprocess.run(["python", "-m", "pydoc", f"cv2.{name}"], stdout=subprocess.PIPE)
    help_text_lines = result.stdout.decode().splitlines()

    if not len(help_text_lines) >= 4:
        return

    function_stubs: list[str] = []
    function_stubs.append("")
    function_stubs.append(f"def {process_function_signature(help_text_lines[3].lstrip())}:")
    function_stubs.append('    """')

    # Pre-process all the lines to make it easier to handle newlines later.
    for i in range(4, len(help_text_lines)):
        help_text_lines[i] = help_text_lines[i].replace("    .    * ", "")
        help_text_lines[i] = help_text_lines[i].replace("    .    *", "")
        help_text_lines[i] = help_text_lines[i].replace("    .   ", "")

    for line in help_text_lines[4:]:
        if re.match(r"    [a-zA-Z].*", line):
            # Remove any blank lines at the end of the docstring.
            while function_stubs[-1].strip() == "":
                function_stubs.pop()
            function_stubs.append('    """')
            function_stubs.append("")
            function_stubs.append("@overload")
            if function_stubs[1] != "@overload":
                function_stubs.insert(1, "@overload")
            function_stubs.append(f"def {process_function_signature(help_text_lines[3].lstrip())}:")
            function_stubs.append('    """')
        else:
            if line == "":
                function_stubs.append("")
            else:
                function_stubs.append("    " + line)

    # Remove any blank lines at the end of the docstring.
    while function_stubs[-1].strip() == "":
        function_stubs.pop()

    function_stubs.append('    """')

    stubs.extend(function_stubs)



from typing import Any, TypeAlias

from .. import functions as cv2

tvec: TypeAlias = Any
img: TypeAlias = Any
rmsd: TypeAlias = Any
pts3d: TypeAlias = Any
bundle: TypeAlias = Any
cols: TypeAlias = Any
pts2d: TypeAlias = Any
ctl2d: TypeAlias = Any
srcLocations: TypeAlias = Any
response: TypeAlias = Any
ctl3d: TypeAlias = Any
rvec: TypeAlias = Any

retval: TypeAlias = Any

class GOSTracker(Tracker):
    def create(self, pts3d, tris, histBins=..., sobelThesh=...) -> retval:
        """"""

class OLSTracker(Tracker):
    def create(self, pts3d, tris, histBins=..., sobelThesh=...) -> retval:
        """"""

class Rapid(Tracker):
    def create(self, pts3d, tris) -> retval:
        """"""

class Tracker(cv2.Algorithm):
    def clearState(self) -> None:
        """"""

    def compute(self, img, num, len, K, rvec, tvec, termcrit=...) -> tuple[retval, rvec, tvec]:
        """"""

def GOSTracker_create(pts3d, tris, histBins=..., sobelThesh=...) -> retval:
    """
    .
    """

def OLSTracker_create(pts3d, tris, histBins=..., sobelThesh=...) -> retval:
    """
    .
    """

def Rapid_create(pts3d, tris) -> retval:
    """
    .
    """

def convertCorrespondencies(cols, srcLocations, pts2d=..., pts3d=..., mask=...) -> tuple[pts2d, pts3d]:
    """
    * Collect corresponding 2d and 3d points based on correspondencies and mask
    @param cols correspondence-position per line in line-bundle-space
    @param srcLocations the source image location
    @param pts2d 2d points
    @param pts3d 3d points
    @param mask mask containing non-zero values for the elements to be retained
    """

def drawCorrespondencies(bundle, cols, colors=...) -> bundle:
    """
    * Debug draw markers of matched correspondences onto a lineBundle
    @param bundle the lineBundle
    @param cols column coordinates in the line bundle
    @param colors colors for the markers. Defaults to white.
    """

def drawSearchLines(img, locations, color) -> img:
    """
    * Debug draw search lines onto an image
    @param img the output image
    @param locations the source locations of a line bundle
    @param color the line color
    """

def drawWireframe(img, pts2d, tris, color, type=..., cullBackface=...) -> img:
    """
    * Draw a wireframe of a triangle mesh
    @param img the output image
    @param pts2d the 2d points obtained by @ref projectPoints
    @param tris triangle face connectivity
    @param color line color
    @param type line type. See @ref LineTypes.
    @param cullBackface enable back-face culling based on CCW order
    """

def extractControlPoints(num, len, pts3d, rvec, tvec, K, imsize, tris, ctl2d=..., ctl3d=...) -> tuple[ctl2d, ctl3d]:
    """
    * Extract control points from the projected silhouette of a mesh

    see @cite drummond2002real Sec 2.1, Step b
    @param num number of control points
    @param len search radius (used to restrict the ROI)
    @param pts3d the 3D points of the mesh
    @param rvec rotation between mesh and camera
    @param tvec translation between mesh and camera
    @param K camera intrinsic
    @param imsize size of the video frame
    @param tris triangle face connectivity
    @param ctl2d the 2D locations of the control points
    @param ctl3d matching 3D points of the mesh
    """

def extractLineBundle(len, ctl2d, img, bundle=..., srcLocations=...) -> tuple[bundle, srcLocations]:
    """
    * Extract the line bundle from an image
    @param len the search radius. The bundle will have `2*len + 1` columns.
    @param ctl2d the search lines will be centered at this points and orthogonal to the contour defined by
    them. The bundle will have as many rows.
    @param img the image to read the pixel intensities values from
    @param bundle line bundle image with size `ctl2d.rows() x (2 * len + 1)` and the same type as @p img
    @param srcLocations the source pixel locations of @p bundle in @p img as CV_16SC2
    """

def findCorrespondencies(bundle, cols=..., response=...) -> tuple[cols, response]:
    """
    * Find corresponding image locations by searching for a maximal sobel edge along the search line (a single
    row in the bundle)
    @param bundle the line bundle
    @param cols correspondence-position per line in line-bundle-space
    @param response the sobel response for the selected point
    """

def rapid(img, num, len, pts3d, tris, K, rvec, tvec) -> tuple[retval, rvec, tvec, rmsd]:
    """
    * High level function to execute a single rapid @cite harris1990rapid iteration

    1. @ref extractControlPoints
    2. @ref extractLineBundle
    3. @ref findCorrespondencies
    4. @ref convertCorrespondencies
    5. @ref solvePnPRefineLM

    @param img the video frame
    @param num number of search lines
    @param len search line radius
    @param pts3d the 3D points of the mesh
    @param tris triangle face connectivity
    @param K camera matrix
    @param rvec rotation between mesh and camera. Input values are used as an initial solution.
    @param tvec translation between mesh and camera. Input values are used as an initial solution.
    @param rmsd the 2d reprojection difference
    @return ratio of search lines that could be extracted and matched
    """



from typing import Any, TypeAlias

version_ABI: TypeAlias = Any
version_API: TypeAlias = Any

retval: TypeAlias = Any

def getBackendName(api) -> retval:
    """
    @brief Returns backend API name or "UnknownVideoAPI(xxx)"
    @param api backend ID (#VideoCaptureAPIs)
    """

def getBackends() -> retval:
    """
    @brief Returns list of all available backends
    """

def getCameraBackendPluginVersion(api) -> tuple[retval, version_ABI, version_API]:
    """
    @brief Returns description and ABI/API version of videoio plugin's camera interface
    """

def getCameraBackends() -> retval:
    """
    @brief Returns list of available backends which works via `cv::VideoCapture(int index)`
    """

def getStreamBackendPluginVersion(api) -> tuple[retval, version_ABI, version_API]:
    """
    @brief Returns description and ABI/API version of videoio plugin's stream capture interface
    """

def getStreamBackends() -> retval:
    """
    @brief Returns list of available backends which works via `cv::VideoCapture(filename)`
    """

def getWriterBackendPluginVersion(api) -> tuple[retval, version_ABI, version_API]:
    """
    @brief Returns description and ABI/API version of videoio plugin's writer interface
    """

def getWriterBackends() -> retval:
    """
    @brief Returns list of available backends which works via `cv::VideoWriter()`
    """

def hasBackend(api) -> retval:
    """
    @brief Returns true if backend is available
    """

def isBackendBuiltIn(api) -> retval:
    """
    @brief Returns true if backend is built in (false if backend is used as plugin)
    """



import builtins
from typing import Any, Final, overload, TypeAlias

retval: TypeAlias = Any

class PyParams:
    def cfgBatchSize(self, size) -> retval:
        """"""

    def cfgNumRequests(self, nireq) -> retval:
        """"""

    def constInput(self, layer_name, data, hint=...) -> retval:
        """"""

@overload
def params(tag, model, weights, device) -> retval:
    """ """

@overload
def params(tag, model, weights, device) -> retval:
    """
    .
    """

ASYNC: Final[int] = 1
Async: Final[int] = 1
SYNC: Final[int] = 0
Sync: Final[int] = 0
TRAIT_AS_IMAGE: Final[int] = 1
TRAIT_AS_TENSOR: Final[int] = 0
TraitAs_IMAGE: Final[int] = 1
TraitAs_TENSOR: Final[int] = 0



from typing import Any, Final, TypeAlias

from .. import functions as cv2

wrappedPhaseMap: TypeAlias = Any
whiteImage: TypeAlias = Any
patternImages: TypeAlias = Any
disparityMap: TypeAlias = Any
dataModulationTerm: TypeAlias = Any
shadowMask: TypeAlias = Any
unwrappedPhaseMap: TypeAlias = Any
projPix: TypeAlias = Any
matches: TypeAlias = Any
blackImage: TypeAlias = Any
retval: TypeAlias = Any

class GrayCodePattern(StructuredLightPattern):
    def getImagesForShadowMasks(self, blackImage, whiteImage) -> tuple[blackImage, whiteImage]:
        """
        @brief Generates the all-black and all-white images needed for shadowMasks computation.
        *
        *  To identify shadow regions, the regions of two images where the pixels are not lit by projector's light and thus where there is not coded information,
        *  the 3DUNDERWORLD algorithm computes a shadow mask for the two cameras views, starting from a white and a black images captured by each camera.
        *  This method generates these two additional images to project.
        *
        *  @param blackImage The generated all-black CV_8U image, at projector's resolution.
        *  @param whiteImage The generated all-white CV_8U image, at projector's resolution.
        """

    def getNumberOfPatternImages(self) -> retval:
        """
        @brief Get the number of pattern images needed for the graycode pattern.
        *
        * @return The number of pattern images needed for the graycode pattern.
        *
        """

    def getProjPixel(self, patternImages, x, y) -> tuple[retval, projPix]:
        """
        @brief For a (x,y) pixel of a camera returns the corresponding projector pixel.
        *
        *  The function decodes each pixel in the pattern images acquired by a camera into their corresponding decimal numbers representing the projector's column and row,
        *  providing a mapping between camera's and projector's pixel.
        *
        *  @param patternImages The pattern images acquired by the camera, stored in a grayscale vector < Mat >.
        *  @param x x coordinate of the image pixel.
        *  @param y y coordinate of the image pixel.
        *  @param projPix Projector's pixel corresponding to the camera's pixel: projPix.x and projPix.y are the image coordinates of the projector's pixel corresponding to the pixel being decoded in a camera.
        """

    def setBlackThreshold(self, value) -> None:
        """
        @brief Sets the value for black threshold, needed for decoding (shadowsmasks computation).
        *
        *  Black threshold is a number between 0-255 that represents the minimum brightness difference required for valid pixels, between the fully illuminated (white) and the not illuminated images (black); used in computeShadowMasks method.
        *
        *  @param value The desired black threshold value. *
        """

    def setWhiteThreshold(self, value) -> None:
        """
        @brief Sets the value for white threshold, needed for decoding.
        *
        *  White threshold is a number between 0-255 that represents the minimum brightness difference required for valid pixels, between the graycode pattern and its inverse images; used in getProjPixel method.
        *
        *  @param value The desired white threshold value. *
        """

    def create(self, width, height) -> retval:
        """
        @brief Constructor
        @param parameters GrayCodePattern parameters GrayCodePattern::Params: the width and the height of the projector.
        """

class SinusoidalPattern(StructuredLightPattern):
    def computeDataModulationTerm(self, patternImages, shadowMask, dataModulationTerm=...) -> dataModulationTerm:
        """
        * @brief compute the data modulation term.
        * @param patternImages captured images with projected patterns.
        * @param dataModulationTerm Mat where the data modulation term is saved.
        * @param shadowMask Mask used to discard shadow regions.
        """

    def computePhaseMap(self, patternImages, wrappedPhaseMap=..., shadowMask=..., fundamental=...) -> tuple[wrappedPhaseMap, shadowMask]:
        """
        * @brief Compute a wrapped phase map from sinusoidal patterns.
        * @param patternImages Input data to compute the wrapped phase map.
        * @param wrappedPhaseMap Wrapped phase map obtained through one of the three methods.
        * @param shadowMask Mask used to discard shadow regions.
        * @param fundamental Fundamental matrix used to compute epipolar lines and ease the matching step.
        """

    def findProCamMatches(self, projUnwrappedPhaseMap, camUnwrappedPhaseMap, matches=...) -> matches:
        """
        * @brief Find correspondences between the two devices thanks to unwrapped phase maps.
        * @param projUnwrappedPhaseMap Projector's unwrapped phase map.
        * @param camUnwrappedPhaseMap Camera's unwrapped phase map.
        * @param matches Images used to display correspondences map.
        """

    def unwrapPhaseMap(self, wrappedPhaseMap, camSize, unwrappedPhaseMap=..., shadowMask=...) -> unwrappedPhaseMap:
        """
        * @brief Unwrap the wrapped phase map to remove phase ambiguities.
        * @param wrappedPhaseMap The wrapped phase map computed from the pattern.
        * @param unwrappedPhaseMap The unwrapped phase map used to find correspondences between the two devices.
        * @param camSize Resolution of the camera.
        * @param shadowMask Mask used to discard shadow regions.
        """

    def create(self, parameters=...) -> retval:
        """
        * @brief Constructor.
        * @param parameters SinusoidalPattern parameters SinusoidalPattern::Params: width, height of the projector and patterns parameters. *
        """

class StructuredLightPattern(cv2.Algorithm):
    def decode(self, patternImages, disparityMap=..., blackImages=..., whiteImages=..., flags=...) -> tuple[retval, disparityMap]:
        """
        @brief Decodes the structured light pattern, generating a disparity map

        @param patternImages The acquired pattern images to decode (vector<vector<Mat>>), loaded as grayscale and previously rectified.
        @param disparityMap The decoding result: a CV_64F Mat at image resolution, storing the computed disparity map.
        @param blackImages The all-black images needed for shadowMasks computation.
        @param whiteImages The all-white images needed for shadowMasks computation.
        @param flags Flags setting decoding algorithms. Default: DECODE_3D_UNDERWORLD. @note All the images must be at the same resolution.
        """

    def generate(self, patternImages=...) -> tuple[retval, patternImages]:
        """
        @brief Generates the structured light pattern to project.

        @param patternImages The generated pattern: a vector<Mat>, in which each image is a CV_8U Mat at projector's resolution.
        """

def GrayCodePattern_create(width, height) -> retval:
    """
    @brief Constructor
       @param parameters GrayCodePattern parameters GrayCodePattern::Params: the width and the height of the projector.
    """

def SinusoidalPattern_create(parameters=...) -> retval:
    """
    * @brief Constructor.
         * @param parameters SinusoidalPattern parameters SinusoidalPattern::Params: width, height of the projector and patterns parameters.
         *
    """

DECODE_3D_UNDERWORLD: Final[int]
FAPS: Final[int]
FTP: Final[int]
PSP: Final[int]



from typing import Final

COLOR_CAMERA_PARAMS_BOARD_SOCKET_BGR: Final[int]
COLOR_CAMERA_PARAMS_BOARD_SOCKET_RGB: Final[int]
COLOR_CAMERA_PARAMS_RESOLUTION_THE_1080_P: int
ColorCameraParams_BoardSocket_BGR: Final[int]
ColorCameraParams_BoardSocket_RGB: Final[int]
ColorCameraParams_Resolution_THE_1080_P: int
ENCODER_CONFIG_PROFILE_H264_BASELINE: Final[int]
ENCODER_CONFIG_PROFILE_H264_HIGH: Final[int]
ENCODER_CONFIG_PROFILE_H264_MAIN: Final[int]
ENCODER_CONFIG_PROFILE_H265_MAIN: Final[int]
ENCODER_CONFIG_PROFILE_MJPEG: Final[int]
ENCODER_CONFIG_RATE_CONTROL_MODE_CBR: Final[int]
ENCODER_CONFIG_RATE_CONTROL_MODE_VBR: Final[int]
EncoderConfig_Profile_H264_BASELINE: Final[int]
EncoderConfig_Profile_H264_HIGH: Final[int]
EncoderConfig_Profile_H264_MAIN: Final[int]
EncoderConfig_Profile_H265_MAIN: Final[int]
EncoderConfig_Profile_MJPEG: Final[int]
EncoderConfig_RateControlMode_CBR: Final[int]
EncoderConfig_RateControlMode_VBR: Final[int]



from typing import Any, overload, TypeAlias

output: TypeAlias = Any

def BIMEF(input, output=..., mu=..., a=..., b=...) -> output:
    """
    * @brief Given an input color image, enhance low-light images using the BIMEF method (@cite ying2017bio @cite ying2017new).

    @param input input color image.
    @param output resulting image.
    @param mu enhancement ratio.
    @param a a-parameter in the Camera Response Function (CRF).
    @param b b-parameter in the Camera Response Function (CRF).

    @warning This is a C++ implementation of the [original MATLAB algorithm](https://github.com/baidut/BIMEF).
    Compared to the original code, this implementation is a little bit slower and does not provide the same results.
    In particular, quality of the image enhancement is degraded for the bright areas in certain conditions.
    """

def BIMEF2(input, k, mu, a, b, output=...) -> output:
    """
    * @brief Given an input color image, enhance low-light images using the BIMEF method (@cite ying2017bio @cite ying2017new).

    This is an overloaded function with the exposure ratio given as parameter.

    @param input input color image.
    @param output resulting image.
    @param k exposure ratio.
    @param mu enhancement ratio.
    @param a a-parameter in the Camera Response Function (CRF).
    @param b b-parameter in the Camera Response Function (CRF).

    @warning This is a C++ implementation of the [original MATLAB algorithm](https://github.com/baidut/BIMEF).
    Compared to the original code, this implementation is a little bit slower and does not provide the same results.
    In particular, quality of the image enhancement is degraded for the bright areas in certain conditions.
    """

def autoscaling(input, output) -> None:
    """
    * @brief Given an input bgr or grayscale image, apply autoscaling on domain [0, 255] to increase
    the contrast of the input image and return the resulting image.

    @param input input bgr or grayscale image.
    @param output resulting image of autoscaling.
    """

def contrastStretching(input, output, r1, s1, r2, s2) -> None:
    """
    * @brief Given an input bgr or grayscale image, apply linear contrast stretching on domain [0, 255]
    and return the resulting image.

    @param input input bgr or grayscale image.
    @param output resulting image of contrast stretching.
    @param r1 x coordinate of first point (r1, s1) in the transformation function.
    @param s1 y coordinate of first point (r1, s1) in the transformation function.
    @param r2 x coordinate of second point (r2, s2) in the transformation function.
    @param s2 y coordinate of second point (r2, s2) in the transformation function.
    """

def gammaCorrection(input, output, gamma) -> None:
    """
    * @brief Given an input bgr or grayscale image and constant gamma, apply power-law transformation,
    a.k.a. gamma correction to the image on domain [0, 255] and return the resulting image.

    @param input input bgr or grayscale image.
    @param output resulting image of gamma corrections.
    @param gamma constant in c*r^gamma where r is pixel value.
    """

def logTransform(input, output) -> None:
    """
    * @brief Given an input bgr or grayscale image and constant c, apply log transformation to the image
    on domain [0, 255] and return the resulting image.

    @param input input bgr or grayscale image.
    @param output resulting image of log transformations.
    """



import builtins
from typing import Any, TypeAlias

result: TypeAlias = Any

retval: TypeAlias = Any

class DnnSuperResImpl:
    def getAlgorithm(self) -> retval:
        """
        @brief Returns the scale factor of the model:
        @return Current algorithm.
        """

    def getScale(self) -> retval:
        """
        @brief Returns the scale factor of the model:
        @return Current scale factor.
        """

    def readModel(self, path) -> None:
        """
        @brief Read the model from the given path
        @param path Path to the model file.
        """

    def setModel(self, algo, scale) -> None:
        """
        @brief Set desired model
        @param algo String containing one of the desired models: - __edsr__ - __espcn__ - __fsrcnn__ - __lapsrn__
        @param scale Integer specifying the upscale factor
        """

    def setPreferableBackend(self, backendId) -> None:
        """
        @brief Set computation backend
        """

    def setPreferableTarget(self, targetId) -> None:
        """
        @brief Set computation target
        """

    def upsample(self, img, result=...) -> result:
        """
        @brief Upsample via neural network
        @param img Image to upscale
        @param result Destination upscaled image
        """

    def upsampleMultioutput(self, img, imgs_new, scale_factors, node_names) -> None:
        """
        @brief Upsample via neural network of multiple outputs
        @param img Image to upscale
        @param imgs_new Destination upscaled images
        @param scale_factors Scaling factors of the output nodes
        @param node_names Names of the output nodes in the neural network
        """

    def create(self) -> retval:
        """
        @brief Empty constructor for python
        """

def DnnSuperResImpl_create() -> retval:
    """
    @brief Empty constructor for python
    """
